# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Dataset containing polar questions and indirect answers."""


import csv

import datasets


_CITATION = """\
@InProceedings{louis_emnlp2020,
  author =      "Annie Louis and Dan Roth and Filip Radlinski",
  title =       ""{I}'d rather just go to bed": {U}nderstanding {I}ndirect {A}nswers",
  booktitle =   "Proceedings of the 2020 Conference on Empirical Methods
  in Natural Language Processing",
  year =        "2020",
}
"""

_DESCRIPTION = """\
The Circa (meaning ‘approximately’) dataset aims to help machine learning systems
to solve the problem of interpreting indirect answers to polar questions.

The dataset contains pairs of yes/no questions and indirect answers, together with
annotations for the interpretation of the answer. The data is collected in 10
different social conversational situations (eg. food preferences of a friend).

NOTE: There might be missing labels in the dataset and we have replaced them with -1.
The original dataset contains no train/dev/test splits.
"""

_LICENSE = "Creative Commons Attribution 4.0 License"

_DATA_URL = "https://raw.githubusercontent.com/google-research-datasets/circa/main/circa-data.tsv"


class Circa(datasets.GeneratorBasedBuilder):
    """Dataset containing polar questions and indirect answers."""

    VERSION = datasets.Version("1.1.0")

    def _info(self):
        features = datasets.Features(
            {
                "context": datasets.Value("string"),
                "question-X": datasets.Value("string"),
                "canquestion-X": datasets.Value("string"),
                "answer-Y": datasets.Value("string"),
                "judgements": datasets.Value("string"),
                "goldstandard1": datasets.features.ClassLabel(
                    names=[
                        "Yes",
                        "No",
                        "In the middle, neither yes nor no",
                        "Probably yes / sometimes yes",
                        "Probably no",
                        "Yes, subject to some conditions",
                        "Other",
                        "I am not sure how X will interpret Y’s answer",
                    ]
                ),
                "goldstandard2": datasets.features.ClassLabel(
                    names=[
                        "Yes",
                        "No",
                        "In the middle, neither yes nor no",
                        "Yes, subject to some conditions",
                        "Other",
                    ]
                ),
            }
        )
        return datasets.DatasetInfo(
            # This is the description that will appear on the datasets page.
            description=_DESCRIPTION,
            # This defines the different columns of the dataset and their types
            features=features,  # Here we define them above because they are different between the two configurations
            # If there's a common (input, target) tuple from the features,
            # specify them here. They'll be used if as_supervised=True in
            # builder.as_dataset.
            supervised_keys=None,
            # Homepage of the dataset for documentation
            homepage="https://github.com/google-research-datasets/circa",
            # License for the dataset if available
            license=_LICENSE,
            # Citation for the dataset
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        train_path = dl_manager.download_and_extract(_DATA_URL)
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                # These kwargs will be passed to _generate_examples
                gen_kwargs={
                    "filepath": train_path,
                    "split": datasets.Split.TRAIN,
                },
            ),
        ]

    def _generate_examples(self, filepath, split):
        with open(filepath, encoding="utf-8") as f:
            goldstandard1_labels = [
                "Yes",
                "No",
                "In the middle, neither yes nor no",
                "Probably yes / sometimes yes",
                "Probably no",
                "Yes, subject to some conditions",
                "Other",
                "I am not sure how X will interpret Y’s answer",
            ]
            goldstandard2_labels = [
                "Yes",
                "No",
                "In the middle, neither yes nor no",
                "Yes, subject to some conditions",
                "Other",
            ]
            data = csv.reader(f, delimiter="\t")
            next(data, None)  # skip the headers
            for id_, row in enumerate(data):
                row = [x if x != "nan" else -1 for x in row]
                _, context, question_X, canquestion_X, answer_Y, judgements, goldstandard1, goldstandard2 = row
                if goldstandard1 not in goldstandard1_labels:
                    goldstandard1 = -1
                if goldstandard2 not in goldstandard2_labels:
                    goldstandard2 = -1

                yield id_, {
                    "context": context,
                    "question-X": question_X,
                    "canquestion-X": canquestion_X,
                    "answer-Y": answer_Y,
                    "judgements": judgements,
                    "goldstandard1": goldstandard1,
                    "goldstandard2": goldstandard2,
                }



# coding=utf-8
# Lint as: python3
"""OffensEval-TR 2020: A Corpus of Turkish Offensive Language on Social Media"""


import csv
import os

import datasets


logger = datasets.logging.get_logger(__name__)


_CITATION = """\
@InProceedings{coltekin2020lrec,
 author  = {Cagri Coltekin},
 year  = {2020},
 title  = {A Corpus of Turkish Offensive Language on Social Media},
 booktitle  = {Proceedings of The 12th Language Resources and Evaluation Conference},
 pages  = {6174--6184},
 address  = {Marseille, France},
 url  = {https://www.aclweb.org/anthology/2020.lrec-1.758},
}
"""


_DESCRIPTION = """\
OffensEval-TR 2020 is a Turkish offensive language corpus. The corpus consist of randomly sampled tweets and annotated in a similar way to OffensEval and GermEval.
"""

_HOMEPAGE = "https://coltekin.github.io/offensive-turkish/"
_DOWNLOAD_URL = "https://coltekin.github.io/offensive-turkish/offenseval2020-turkish.zip"
_FOLDER_NAME = "offenseval-tr-{split}-v1"


class OffensEval2020TRConfig(datasets.BuilderConfig):
    """BuilderConfig for OffensEval2020TR."""

    def __init__(self, **kwargs):
        """BuilderConfig for OffensEval2020TR.

        Args:
          **kwargs: keyword arguments forwarded to super.
        """
        super(OffensEval2020TRConfig, self).__init__(**kwargs)


class Offenseval2020TR(datasets.GeneratorBasedBuilder):
    """OffensEval-TR 2020: A Corpus of Turkish Offensive Language on Social Media"""

    BUILDER_CONFIGS = [
        OffensEval2020TRConfig(
            name="offenseval2020-turkish",
            version=datasets.Version("1.0.0"),
            description="OffensEval-TR 2020: A Corpus of Turkish Offensive Language on Social Media",
        ),
    ]

    def _info(self):
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(
                {
                    "id": datasets.Value("int32"),
                    "tweet": datasets.Value("string"),
                    "subtask_a": datasets.features.ClassLabel(names=["NOT", "OFF"]),
                }
            ),
            supervised_keys=None,
            homepage=_HOMEPAGE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        dl_dir = dl_manager.download_and_extract(_DOWNLOAD_URL)
        data_dir = os.path.join(dl_dir, self.config.name)

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": os.path.join(
                        data_dir, _FOLDER_NAME.format(split="training"), "offenseval-tr-training-v1.tsv"
                    ),
                    "labelpath": None,
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={
                    "filepath": os.path.join(
                        data_dir, _FOLDER_NAME.format(split="testset"), "offenseval-tr-testset-v1.tsv"
                    ),
                    "labelpath": os.path.join(
                        data_dir, _FOLDER_NAME.format(split="testset"), "offenseval-tr-labela-v1.tsv"
                    ),
                },
            ),
        ]

    def _generate_examples(self, filepath, labelpath):
        """Generate OffensEval2020TR examples."""
        logger.info("⏳ Generating examples from = %s", filepath)

        if labelpath:
            with open(filepath, encoding="utf-8") as f:
                with open(labelpath, encoding="utf-8") as f2:
                    reader_testset = csv.DictReader(f, delimiter="\t", quoting=csv.QUOTE_NONE)
                    reader_label = csv.DictReader(
                        f2, delimiter=",", quoting=csv.QUOTE_NONE, fieldnames=["id", "subtask_a"]
                    )
                    list_label = list(reader_label)
                    for idx, row in enumerate(reader_testset):
                        row_label = list_label[idx]
                        yield idx, {"id": row["id"], "tweet": row["tweet"], "subtask_a": row_label["subtask_a"]}
        else:
            with open(filepath, encoding="utf-8") as f:
                reader = csv.DictReader(f, delimiter="\t", quoting=csv.QUOTE_NONE)
                for idx, row in enumerate(reader):
                    yield idx, {"id": row["id"], "tweet": row["tweet"], "subtask_a": row["subtask_a"]}



# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""AQUA-RAT (Algebra Question Answering with Rationales) Dataset"""


import json

import datasets


_CITATION = """\
@InProceedings{ACL,
title = {Program induction by rationale generation: Learning to solve and explain algebraic word problems},
authors={Ling, Wang and Yogatama, Dani and Dyer, Chris and Blunsom, Phil},
year={2017}
}
"""

_DESCRIPTION = """\
A large-scale dataset consisting of approximately 100,000 algebraic word problems.
The solution to each question is explained step-by-step using natural language.
This data is used to train a program generation model that learns to generate the explanation,
while generating the program that solves the question.
"""

_LICENSE = """\
Copyright 2017 Google Inc.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

_BASE_URL = "https://raw.githubusercontent.com/deepmind/AQuA/master/"
_URLs = {
    "raw": {"train": _BASE_URL + "train.json", "dev": _BASE_URL + "dev.json", "test": _BASE_URL + "test.json"},
    "tokenized": {
        "train": _BASE_URL + "train.tok.json",
        "dev": _BASE_URL + "dev.tok.json",
        "test": _BASE_URL + "test.tok.json",
    },
}


class AquaRat(datasets.GeneratorBasedBuilder):
    """AQUA-RAT (Algebra Question Answering with Rationales) Dataset"""

    VERSION = datasets.Version("1.0.0")

    BUILDER_CONFIGS = [
        datasets.BuilderConfig(name="raw", description="Untokenized Dataset"),
        datasets.BuilderConfig(name="tokenized", description="Tokenized Dataset"),
    ]

    DEFAULT_CONFIG_NAME = "raw"

    def _info(self):
        features = datasets.Features(
            {
                "question": datasets.Value("string"),
                "options": datasets.features.Sequence(datasets.Value("string")),
                "rationale": datasets.Value("string"),
                "correct": datasets.Value("string"),
            }
        )
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            supervised_keys=None,
            homepage="https://github.com/deepmind/AQuA",
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""
        my_urls = _URLs[self.config.name]
        data_paths = dl_manager.download(my_urls)
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": data_paths["train"],
                    "split": "train",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={"filepath": data_paths["test"], "split": "test"},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={
                    "filepath": data_paths["dev"],
                    "split": "dev",
                },
            ),
        ]

    def _generate_examples(self, filepath, split):
        """Yields examples."""
        with open(filepath, encoding="utf-8") as f:
            for id_, row in enumerate(f):
                data = json.loads(row)
                yield id_, {
                    "question": data["question"],
                    "options": data["options"],
                    "rationale": data["rationale"],
                    "correct": data["correct"],
                }



# coding=utf-8
# Copyright 2020 HuggingFace Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Lint as: python3
""""The Finnish News Corpus for Named Entity Recognition dataset."""


import csv

import datasets


logger = datasets.logging.get_logger(__name__)


_CITATION = """\
@article{ruokolainen2019finnish,
  title={A finnish news corpus for named entity recognition},
  author={Ruokolainen, Teemu and Kauppinen, Pekka and Silfverberg, Miikka and Lind{\'e}n, Krister},
  journal={Language Resources and Evaluation},
  pages={1--26},
  year={2019},
  publisher={Springer}
}
"""

_DESCRIPTION = """\
The directory data contains a corpus of Finnish technology related news articles with a manually prepared
named entity annotation (digitoday.2014.csv). The text material was extracted from the archives of Digitoday,
a Finnish online technology news source (www.digitoday.fi). The corpus consists of 953 articles
(193,742 word tokens) with six named entity classes (organization, location, person, product, event, and date).
The corpus is available for research purposes and can be readily used for development of NER systems for Finnish.
"""

_URLS = {
    "train": "https://github.com/mpsilfve/finer-data/raw/master/data/digitoday.2014.train.csv",
    "dev": "https://github.com/mpsilfve/finer-data/raw/master/data/digitoday.2014.dev.csv",
    "test": "https://github.com/mpsilfve/finer-data/raw/master/data/digitoday.2015.test.csv",
    "test_wikipedia": "https://github.com/mpsilfve/finer-data/raw/master/data/wikipedia.test.csv",
}


class FinerConfig(datasets.BuilderConfig):
    """BuilderConfig for FiNER dataset."""

    def __init__(self, **kwargs):
        """BuilderConfig for FiNER dataset.

        Args:
          **kwargs: keyword arguments forwarded to super.
        """
        super(FinerConfig, self).__init__(**kwargs)


class Finer(datasets.GeneratorBasedBuilder):
    """FiNER dataset."""

    BUILDER_CONFIGS = [
        FinerConfig(
            name="finer",
            version=datasets.Version("1.0.0"),
            description="A Finnish News Corpus for Named Entity Recognition dataset",
        ),
    ]

    def _info(self):
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(
                {
                    "id": datasets.Value("string"),
                    "tokens": datasets.Sequence(datasets.Value("string")),
                    "ner_tags": datasets.Sequence(
                        datasets.features.ClassLabel(
                            names=[
                                "O",
                                "B-DATE",
                                "B-EVENT",
                                "B-LOC",
                                "B-ORG",
                                "B-PER",
                                "B-PRO",
                                "I-DATE",
                                "I-EVENT",
                                "I-LOC",
                                "I-ORG",
                                "I-PER",
                                "I-PRO",
                            ]
                        )
                    ),
                    "nested_ner_tags": datasets.Sequence(
                        datasets.features.ClassLabel(
                            names=[
                                "O",
                                "B-DATE",
                                "B-EVENT",
                                "B-LOC",
                                "B-ORG",
                                "B-PER",
                                "B-PRO",
                                "I-DATE",
                                "I-EVENT",
                                "I-LOC",
                                "I-ORG",
                                "I-PER",
                                "I-PRO",
                            ]
                        )
                    ),
                }
            ),
            supervised_keys=None,
            homepage="https://github.com/mpsilfve/finer-data",
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""
        downloaded_files = dl_manager.download_and_extract(_URLS)

        return [
            datasets.SplitGenerator(name=datasets.Split.TRAIN, gen_kwargs={"filepath": downloaded_files["train"]}),
            datasets.SplitGenerator(name=datasets.Split.VALIDATION, gen_kwargs={"filepath": downloaded_files["dev"]}),
            datasets.SplitGenerator(name=datasets.Split.TEST, gen_kwargs={"filepath": downloaded_files["test"]}),
            datasets.SplitGenerator(
                name=datasets.Split("test_wikipedia"), gen_kwargs={"filepath": downloaded_files["test_wikipedia"]}
            ),
        ]

    def _generate_examples(self, filepath):
        logger.info("⏳ Generating 🇫🇮 examples from = %s", filepath)
        with open(filepath, encoding="utf-8") as f:
            data = csv.reader(f, delimiter="\t", quoting=csv.QUOTE_NONE)
            current_tokens = []
            current_ner_tags = []
            current_nested_ner_tags = []
            sentence_counter = 0
            for row in data:
                if row and "" not in row:
                    token, label, nested_label = row[:3]
                    current_tokens.append(token)
                    current_ner_tags.append(label)
                    current_nested_ner_tags.append(nested_label)
                else:
                    # New sentence
                    if not current_tokens:
                        # Consecutive empty lines will cause empty sentences
                        continue
                    assert len(current_tokens) == len(current_ner_tags), "💔 between len of tokens & labels"
                    assert len(current_ner_tags) == len(
                        current_nested_ner_tags
                    ), "💔 between len of labels & nested labels"
                    sentence = (
                        sentence_counter,
                        {
                            "id": str(sentence_counter),
                            "tokens": current_tokens,
                            "ner_tags": current_ner_tags,
                            "nested_ner_tags": current_nested_ner_tags,
                        },
                    )
                    sentence_counter += 1
                    current_tokens = []
                    current_ner_tags = []
                    current_nested_ner_tags = []
                    yield sentence
            # Don't forget last sentence in dataset 🧐
            if current_tokens:
                yield sentence_counter, {
                    "id": str(sentence_counter),
                    "tokens": current_tokens,
                    "ner_tags": current_ner_tags,
                    "nested_ner_tags": current_nested_ner_tags,
                }



# coding=utf-8
# Copyright 2020 The TensorFlow Datasets Authors and the HuggingFace Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Wiki40B: A clean Wikipedia dataset for 40+ languages."""


import datasets


logger = datasets.logging.get_logger(__name__)


_CITATION = """
"""

_DESCRIPTION = """
Clean-up text for 40+ Wikipedia languages editions of pages
correspond to entities. The datasets have train/dev/test splits per language.
The dataset is cleaned up by page filtering to remove disambiguation pages,
redirect pages, deleted pages, and non-entity pages. Each example contains the
wikidata id of the entity, and the full Wikipedia article after page processing
that removes non-content sections and structured objects.
"""

_LICENSE = """
This work is licensed under the Creative Commons Attribution-ShareAlike
3.0 Unported License. To view a copy of this license, visit
http://creativecommons.org/licenses/by-sa/3.0/ or send a letter to
Creative Commons, PO Box 1866, Mountain View, CA 94042, USA.
"""

_URL = "https://research.google/pubs/pub49029/"

_DATA_DIRECTORY = "gs://tfds-data/downloads/wiki40b/tfrecord_prod"

WIKIPEDIA_LANGUAGES = [
    "en",
    "ar",
    "zh-cn",
    "zh-tw",
    "nl",
    "fr",
    "de",
    "it",
    "ja",
    "ko",
    "pl",
    "pt",
    "ru",
    "es",
    "th",
    "tr",
    "bg",
    "ca",
    "cs",
    "da",
    "el",
    "et",
    "fa",
    "fi",
    "he",
    "hi",
    "hr",
    "hu",
    "id",
    "lt",
    "lv",
    "ms",
    "no",
    "ro",
    "sk",
    "sl",
    "sr",
    "sv",
    "tl",
    "uk",
    "vi",
]


class Wiki40bConfig(datasets.BuilderConfig):
    """BuilderConfig for Wiki40B."""

    def __init__(self, language=None, **kwargs):
        """BuilderConfig for Wiki40B.

        Args:
          language: string, the language code for the Wiki40B dataset to use.
          **kwargs: keyword arguments forwarded to super.
        """
        super(Wiki40bConfig, self).__init__(
            name=str(language), description=f"Wiki40B dataset for {language}.", **kwargs
        )
        self.language = language


_VERSION = datasets.Version("1.1.0")


class Wiki40b(datasets.BeamBasedBuilder):
    """Wiki40B: A Clean Wikipedia Dataset for Mutlilingual Language Modeling."""

    BUILDER_CONFIGS = [
        Wiki40bConfig(
            version=_VERSION,
            language=lang,
        )  # pylint:disable=g-complex-comprehension
        for lang in WIKIPEDIA_LANGUAGES
    ]

    def _info(self):
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(
                {
                    "wikidata_id": datasets.Value("string"),
                    "text": datasets.Value("string"),
                    "version_id": datasets.Value("string"),
                }
            ),
            supervised_keys=None,
            homepage=_URL,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""

        lang = self.config.language

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={"filepaths": f"{_DATA_DIRECTORY}/train/{lang}_examples-*"},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={"filepaths": f"{_DATA_DIRECTORY}/dev/{lang}_examples-*"},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={"filepaths": f"{_DATA_DIRECTORY}/test/{lang}_examples-*"},
            ),
        ]

    def _build_pcollection(self, pipeline, filepaths):
        """Build PCollection of examples."""
        import apache_beam as beam
        import tensorflow as tf

        logger.info("generating examples from = %s", filepaths)

        def _extract_content(example):
            """Extracts content from a TFExample."""
            wikidata_id = example.features.feature["wikidata_id"].bytes_list.value[0].decode("utf-8")
            text = example.features.feature["text"].bytes_list.value[0].decode("utf-8")
            version_id = example.features.feature["version_id"].bytes_list.value[0].decode("utf-8")

            # wikidata_id could be duplicated with different texts.
            yield wikidata_id + text, {
                "wikidata_id": wikidata_id,
                "text": text,
                "version_id": version_id,
            }

        return (
            pipeline
            | beam.io.ReadFromTFRecord(filepaths, coder=beam.coders.ProtoCoder(tf.train.Example))
            | beam.FlatMap(_extract_content)
        )



# coding=utf-8
# Copyright 2020 HuggingFace Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Lint as: python3
"""Introduction to the CoNLL-2003 Shared Task: Language-Independent Named Entity Recognition"""

import os

import datasets


logger = datasets.logging.get_logger(__name__)


_CITATION = """\
@inproceedings{tjong-kim-sang-de-meulder-2003-introduction,
    title = "Introduction to the {C}o{NLL}-2003 Shared Task: Language-Independent Named Entity Recognition",
    author = "Tjong Kim Sang, Erik F.  and
      De Meulder, Fien",
    booktitle = "Proceedings of the Seventh Conference on Natural Language Learning at {HLT}-{NAACL} 2003",
    year = "2003",
    url = "https://www.aclweb.org/anthology/W03-0419",
    pages = "142--147",
}
"""

_DESCRIPTION = """\
The shared task of CoNLL-2003 concerns language-independent named entity recognition. We will concentrate on
four types of named entities: persons, locations, organizations and names of miscellaneous entities that do
not belong to the previous three groups.

The CoNLL-2003 shared task data files contain four columns separated by a single space. Each word has been put on
a separate line and there is an empty line after each sentence. The first item on each line is a word, the second
a part-of-speech (POS) tag, the third a syntactic chunk tag and the fourth the named entity tag. The chunk tags
and the named entity tags have the format I-TYPE which means that the word is inside a phrase of type TYPE. Only
if two phrases of the same type immediately follow each other, the first word of the second phrase will have tag
B-TYPE to show that it starts a new phrase. A word with tag O is not part of a phrase. Note the dataset uses IOB2
tagging scheme, whereas the original dataset uses IOB1.

For more details see https://www.clips.uantwerpen.be/conll2003/ner/ and https://www.aclweb.org/anthology/W03-0419
"""

_URL = "https://data.deepai.org/conll2003.zip"
_TRAINING_FILE = "train.txt"
_DEV_FILE = "valid.txt"
_TEST_FILE = "test.txt"


class Conll2003Config(datasets.BuilderConfig):
    """BuilderConfig for Conll2003"""

    def __init__(self, **kwargs):
        """BuilderConfig forConll2003.

        Args:
          **kwargs: keyword arguments forwarded to super.
        """
        super(Conll2003Config, self).__init__(**kwargs)


class Conll2003(datasets.GeneratorBasedBuilder):
    """Conll2003 dataset."""

    BUILDER_CONFIGS = [
        Conll2003Config(name="conll2003", version=datasets.Version("1.0.0"), description="Conll2003 dataset"),
    ]

    def _info(self):
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(
                {
                    "id": datasets.Value("string"),
                    "tokens": datasets.Sequence(datasets.Value("string")),
                    "pos_tags": datasets.Sequence(
                        datasets.features.ClassLabel(
                            names=[
                                '"',
                                "''",
                                "#",
                                "$",
                                "(",
                                ")",
                                ",",
                                ".",
                                ":",
                                "``",
                                "CC",
                                "CD",
                                "DT",
                                "EX",
                                "FW",
                                "IN",
                                "JJ",
                                "JJR",
                                "JJS",
                                "LS",
                                "MD",
                                "NN",
                                "NNP",
                                "NNPS",
                                "NNS",
                                "NN|SYM",
                                "PDT",
                                "POS",
                                "PRP",
                                "PRP$",
                                "RB",
                                "RBR",
                                "RBS",
                                "RP",
                                "SYM",
                                "TO",
                                "UH",
                                "VB",
                                "VBD",
                                "VBG",
                                "VBN",
                                "VBP",
                                "VBZ",
                                "WDT",
                                "WP",
                                "WP$",
                                "WRB",
                            ]
                        )
                    ),
                    "chunk_tags": datasets.Sequence(
                        datasets.features.ClassLabel(
                            names=[
                                "O",
                                "B-ADJP",
                                "I-ADJP",
                                "B-ADVP",
                                "I-ADVP",
                                "B-CONJP",
                                "I-CONJP",
                                "B-INTJ",
                                "I-INTJ",
                                "B-LST",
                                "I-LST",
                                "B-NP",
                                "I-NP",
                                "B-PP",
                                "I-PP",
                                "B-PRT",
                                "I-PRT",
                                "B-SBAR",
                                "I-SBAR",
                                "B-UCP",
                                "I-UCP",
                                "B-VP",
                                "I-VP",
                            ]
                        )
                    ),
                    "ner_tags": datasets.Sequence(
                        datasets.features.ClassLabel(
                            names=[
                                "O",
                                "B-PER",
                                "I-PER",
                                "B-ORG",
                                "I-ORG",
                                "B-LOC",
                                "I-LOC",
                                "B-MISC",
                                "I-MISC",
                            ]
                        )
                    ),
                }
            ),
            supervised_keys=None,
            homepage="https://www.aclweb.org/anthology/W03-0419/",
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""
        downloaded_file = dl_manager.download_and_extract(_URL)
        data_files = {
            "train": os.path.join(downloaded_file, _TRAINING_FILE),
            "dev": os.path.join(downloaded_file, _DEV_FILE),
            "test": os.path.join(downloaded_file, _TEST_FILE),
        }

        return [
            datasets.SplitGenerator(name=datasets.Split.TRAIN, gen_kwargs={"filepath": data_files["train"]}),
            datasets.SplitGenerator(name=datasets.Split.VALIDATION, gen_kwargs={"filepath": data_files["dev"]}),
            datasets.SplitGenerator(name=datasets.Split.TEST, gen_kwargs={"filepath": data_files["test"]}),
        ]

    def _generate_examples(self, filepath):
        logger.info("⏳ Generating examples from = %s", filepath)
        with open(filepath, encoding="utf-8") as f:
            guid = 0
            tokens = []
            pos_tags = []
            chunk_tags = []
            ner_tags = []
            for line in f:
                if line.startswith("-DOCSTART-") or line == "" or line == "\n":
                    if tokens:
                        yield guid, {
                            "id": str(guid),
                            "tokens": tokens,
                            "pos_tags": pos_tags,
                            "chunk_tags": chunk_tags,
                            "ner_tags": ner_tags,
                        }
                        guid += 1
                        tokens = []
                        pos_tags = []
                        chunk_tags = []
                        ner_tags = []
                else:
                    # conll2003 tokens are space separated
                    splits = line.split(" ")
                    tokens.append(splits[0])
                    pos_tags.append(splits[1])
                    chunk_tags.append(splits[2])
                    ner_tags.append(splits[3].rstrip())
            # last example
            yield guid, {
                "id": str(guid),
                "tokens": tokens,
                "pos_tags": pos_tags,
                "chunk_tags": chunk_tags,
                "ner_tags": ner_tags,
            }



# coding=utf-8
# Copyright 2020 The TensorFlow Datasets Authors and the HuggingFace Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Lint as: python3
"""The French Language Understanding Evaluation (FLUE) benchmark."""


import csv
import os
import re
import textwrap
import unicodedata
from shutil import copyfile

from lxml import etree

import datasets


_FLUE_CITATION = """\
@misc{le2019flaubert,
    title={FlauBERT: Unsupervised Language Model Pre-training for French},
    author={Hang Le and Loïc Vial and Jibril Frej and Vincent Segonne and Maximin Coavoux and Benjamin Lecouteux and Alexandre Allauzen and Benoît Crabbé and Laurent Besacier and Didier Schwab},
    year={2019},
    eprint={1912.05372},
    archivePrefix={arXiv},
    primaryClass={cs.CL}
}
"""

_FLUE_DESCRIPTION = """\
FLUE is an evaluation setup for French NLP systems similar to the popular GLUE benchmark. The goal is to enable further reproducible experiments in the future and to share models and progress on the French language.
"""


class FlueConfig(datasets.BuilderConfig):
    """BuilderConfig for FLUE."""

    def __init__(
        self,
        text_features,
        label_column,
        data_url,
        data_dir,
        citation,
        url,
        label_classes=None,
        process_label=lambda x: x,
        **kwargs,
    ):
        """BuilderConfig for FLUE.

        Args:
          text_features: `dict[string, string]`, map from the name of the feature
            dict for each text field to the name of the column in the tsv file
          label_column: `string`, name of the column in the tsv file corresponding
            to the label
          data_url: `string`, url to download the zip file from
          data_dir: `string`, the path to the folder containing the tsv files in the
            downloaded zip
          citation: `string`, citation for the data set
          url: `string`, url for information about the data set
          label_classes: `list[string]`, the list of classes if the label is
            categorical. If not provided, then the label will be of type
            `datasets.Value('float32')`.
          process_label: `Function[string, any]`, function  taking in the raw value
            of the label and processing it to the form required by the label feature
          **kwargs: keyword arguments forwarded to super.
        """
        super(FlueConfig, self).__init__(version=datasets.Version("1.0.0", ""), **kwargs)
        self.text_features = text_features
        self.label_column = label_column
        self.label_classes = label_classes
        self.data_url = data_url
        self.data_dir = data_dir
        self.citation = citation
        self.url = url
        self.process_label = process_label


class Flue(datasets.GeneratorBasedBuilder):
    """The French Language Understanding Evaluation (FLUE) benchmark."""

    BUILDER_CONFIGS = [
        FlueConfig(
            name="CLS",
            description=textwrap.dedent(
                """\
            This is a binary classification task. It consists in classifying Amazon reviews for three product categories:
            books, DVD, and music. Each sample contains a review text and the associated rating from 1 to 5 stars. Reviews
            rated above 3 is labeled as positive, and those rated less than 3 is labeled as negative. The train and test sets
            are balanced, including around 1k positive and 1k negative reviews for a total of 2k reviews in each dataset. Only
            the French portion is taken to create the binary text classification task in FLUE and report the accuracy on the test set."""
            ),
            text_features={"text": "text"},
            label_classes=["negative", "positive"],
            label_column="label",
            data_url="https://zenodo.org/record/3251672/files/cls-acl10-unprocessed.tar.gz",
            data_dir="",
            url="",
            citation="",
        ),
        FlueConfig(
            name="PAWS-X",
            description=textwrap.dedent(
                """\
            This dataset contains 23,659 human translated PAWS evaluation pairs and 296,406 machine translated training
            pairs in six typologically distinct languages: French, Spanish, German, Chinese, Japanese, and Korean. All
            translated pairs are sourced from examples in PAWS-Wiki. Only the related dataset for French is taken to perform
            the paraphrasing task and report the accuracy on the test set."""
            ),
            text_features={"sentence1": "sentence1", "sentence2": "sentence2"},
            data_url="https://storage.googleapis.com/paws/pawsx/x-final.tar.gz",
            label_column="label",
            data_dir="",
            url="https://github.com/google-research-datasets/paws/tree/master/pawsx",
            citation=textwrap.dedent(
                """\
            @InProceedings{pawsx2019emnlp,
                title = {{PAWS-X: A Cross-lingual Adversarial Dataset for Paraphrase Identification}},
                author = {Yang, Yinfei and Zhang, Yuan and Tar, Chris and Baldridge, Jason},
                booktitle = {Proc. of EMNLP},
                year = {2019}
            }"""
            ),
        ),
        FlueConfig(
            name="XNLI",
            description=textwrap.dedent(
                """
                The Cross-lingual Natural Language Inference (XNLI) corpus is a crowd-sourced collection of 5,000 test and
                2,500 dev pairs for the MultiNLI corpus. The pairs are annotated with textual entailment and translated into
                14 languages: French, Spanish, German, Greek, Bulgarian, Russian, Turkish, Arabic, Vietnamese, Thai, Chinese,
                Hindi, Swahili and Urdu. This results in 112.5k annotated pairs. Each premise can be associated with the
                corresponding hypothesis in the 15 languages, summing up to more than 1.5M combinations. The corpus is made to
                evaluate how to perform inference in any language (including low-resources ones like Swahili or Urdu) when only
                English NLI data is available at training time. One solution is cross-lingual sentence encoding, for which XNLI
                is an evaluation benchmark. Only the related datasets for French is taken to perform the NLI task and report
                the accuracy on the test set."""
            ),
            text_features={"premise": "premise", "hypo": "hypo"},
            data_url={
                "train": "https://dl.fbaipublicfiles.com/XNLI/XNLI-MT-1.0.zip",
                "dev_test": "https://dl.fbaipublicfiles.com/XNLI/XNLI-1.0.zip",
            },
            label_classes=["contradiction", "entailment", "neutral"],
            label_column="label",
            data_dir="",
            url="https://www.nyu.edu/projects/bowman/xnli/",
            citation=textwrap.dedent(
                """\
                @InProceedings{conneau2018xnli,
                author = {Conneau, Alexis
                                and Rinott, Ruty
                                and Lample, Guillaume
                                and Williams, Adina
                                and Bowman, Samuel R.
                                and Schwenk, Holger
                                and Stoyanov, Veselin},
                title = {XNLI: Evaluating Cross-lingual Sentence Representations},
                booktitle = {Proceedings of the 2018 Conference on Empirical Methods
                            in Natural Language Processing},
                year = {2018},
                publisher = {Association for Computational Linguistics},
                location = {Brussels, Belgium},
                }"""
            ),
        ),
        FlueConfig(
            name="WSD-V",
            description=textwrap.dedent(
                """
                French Verb Sense Disambiguation task."""
            ),
            text_features={
                "sentence": "sentence",
                "pos_tags": "pos_tags",
                "lemmas": "lemmas",
                "fine_pos_tags": "fine_pos_tags",
            },
            data_url="http://www.llf.cnrs.fr/dataset/fse/FSE-1.1-10_12_19.tar.gz",
            label_classes=["disambiguate_tokens_ids", "disambiguate_labels"],
            label_column="disambiguate_labels",
            data_dir="FSE-1.1-191210",
            url="http://www.llf.cnrs.fr/dataset/fse/",
            citation="",
        ),
    ]

    def _info(self):
        if self.config.name == "CLS" or self.config.name == "XNLI":
            features = {text_feature: datasets.Value("string") for text_feature in self.config.text_features.keys()}
            features[self.config.label_column] = datasets.features.ClassLabel(names=self.config.label_classes)
            features["idx"] = datasets.Value("int32")
        elif self.config.name == "WSD-V":
            features = {
                text_feature: datasets.Sequence(datasets.Value("string"))
                for text_feature in self.config.text_features.keys()
            }
            features["fine_pos_tags"] = datasets.Sequence(
                datasets.features.ClassLabel(
                    names=[
                        "DET",
                        "P+D",
                        "CC",
                        "VS",
                        "P",
                        "CS",
                        "NC",
                        "NPP",
                        "ADJWH",
                        "VINF",
                        "VPP",
                        "ADVWH",
                        "PRO",
                        "V",
                        "CLO",
                        "PREF",
                        "VPR",
                        "PROREL",
                        "ADV",
                        "PROWH",
                        "N",
                        "DETWH",
                        "ADJ",
                        "P+PRO",
                        "ET",
                        "VIMP",
                        "CLS",
                        "PONCT",
                        "I",
                        "CLR",
                    ]
                )
            )
            features["pos_tags"] = datasets.Sequence(
                datasets.features.ClassLabel(
                    names=[
                        "V",
                        "PREF",
                        "P+D",
                        "I",
                        "A",
                        "P+PRO",
                        "PRO",
                        "P",
                        "anonyme",
                        "D",
                        "C",
                        "CL",
                        "ET",
                        "PONCT",
                        "ADV",
                        "N",
                    ]
                )
            )
            features["disambiguate_tokens_ids"] = datasets.Sequence(datasets.Value("int32"))
            features["disambiguate_labels"] = datasets.Sequence(datasets.Value("string"))
            features["idx"] = datasets.Value("string")
        else:
            features = {text_feature: datasets.Value("string") for text_feature in self.config.text_features.keys()}
            features[self.config.label_column] = datasets.Value("int32")
            features["idx"] = datasets.Value("int32")
        return datasets.DatasetInfo(
            description=_FLUE_DESCRIPTION,
            features=datasets.Features(features),
            homepage=self.config.url,
            citation=self.config.citation + "\n" + _FLUE_CITATION,
        )

    def _split_generators(self, dl_manager):
        if self.config.name == "CLS":
            archive = dl_manager.download(self.config.data_url)

            return [
                datasets.SplitGenerator(
                    name=datasets.Split.TRAIN,
                    gen_kwargs={
                        "data_file": ("cls-acl10-unprocessed", "fr"),
                        "split": "train",
                        "files": dl_manager.iter_archive(archive),
                    },
                ),
                datasets.SplitGenerator(
                    name=datasets.Split.TEST,
                    gen_kwargs={
                        "data_file": ("cls-acl10-unprocessed", "fr"),
                        "split": "test",
                        "files": dl_manager.iter_archive(archive),
                    },
                ),
            ]
        elif self.config.name == "PAWS-X":
            archive = dl_manager.download(self.config.data_url)

            return [
                datasets.SplitGenerator(
                    name=datasets.Split.VALIDATION,
                    gen_kwargs={
                        "data_file": ("x-final", "fr", "dev_2k.tsv"),
                        "split": "",
                        "files": dl_manager.iter_archive(archive),
                    },
                ),
                datasets.SplitGenerator(
                    name=datasets.Split.TEST,
                    gen_kwargs={
                        "data_file": ("x-final", "fr", "test_2k.tsv"),
                        "split": "",
                        "files": dl_manager.iter_archive(archive),
                    },
                ),
                datasets.SplitGenerator(
                    name=datasets.Split.TRAIN,
                    gen_kwargs={
                        "data_file": ("x-final", "fr", "translated_train.tsv"),
                        "split": "",
                        "files": dl_manager.iter_archive(archive),
                    },
                ),
            ]
        elif self.config.name == "XNLI":
            data_folder = dl_manager.download_and_extract(self.config.data_url)
            return [
                datasets.SplitGenerator(
                    name=datasets.Split.VALIDATION,
                    gen_kwargs={
                        "data_file": os.path.join(data_folder["dev_test"], "XNLI-1.0", "xnli.dev.tsv"),
                        "split": "dev",
                    },
                ),
                datasets.SplitGenerator(
                    name=datasets.Split.TEST,
                    gen_kwargs={
                        "data_file": os.path.join(data_folder["dev_test"], "XNLI-1.0", "xnli.test.tsv"),
                        "split": "test",
                    },
                ),
                datasets.SplitGenerator(
                    name=datasets.Split.TRAIN,
                    gen_kwargs={
                        "data_file": os.path.join(
                            data_folder["train"], "XNLI-MT-1.0", "multinli", "multinli.train.fr.tsv"
                        ),
                        "split": "train",
                    },
                ),
            ]
        elif self.config.name == "WSD-V":
            # TODO(QL): make streamable using iter_archive
            data_folder = dl_manager.download_and_extract(self.config.data_url)
            self._wsdv_prepare_data(os.path.join(data_folder, self.config.data_dir))

            return [
                datasets.SplitGenerator(
                    name=datasets.Split.TRAIN,
                    gen_kwargs={
                        "data_file": os.path.join(data_folder, self.config.data_dir),
                        "split": "train",
                    },
                ),
                datasets.SplitGenerator(
                    name=datasets.Split.TEST,
                    gen_kwargs={
                        "data_file": os.path.join(data_folder, self.config.data_dir),
                        "split": "test",
                    },
                ),
            ]

    def _generate_examples(self, data_file, split, files=None):
        if self.config.name == "CLS":
            for path, f in files:
                for category in ["books", "dvd", "music"]:
                    file_path = "/".join(data_file + (category, split + ".review"))
                    if path == file_path:
                        next(f)
                        id = 0
                        text = f.read().decode("utf-8")
                        for id_, line in enumerate(text.split("\n\n")):
                            if len(line) > 9:
                                id += 1
                                review_text, label = self._cls_extractor(line)
                                yield f"{category}_{id_}", {"idx": id, "text": review_text, "label": label}
        elif self.config.name == "PAWS-X":
            for path, f in files:
                if path == "/".join(data_file):
                    data = csv.reader((line.decode("utf-8") for line in f), delimiter="\t")
                    next(data)  # skip header
                    id = 0
                    for id_, row in enumerate(data):
                        if len(row) == 4:
                            id += 1
                            yield id_, {
                                "idx": id,
                                "sentence1": self._cleaner(row[1]),
                                "sentence2": self._cleaner(row[2]),
                                "label": int(row[3].strip()),
                            }
        elif self.config.name == "XNLI":
            with open(data_file, encoding="utf-8") as f:
                data = csv.reader(f, delimiter="\t", quoting=csv.QUOTE_NONE)
                next(data)
                id = 0
                for id_, row in enumerate(data):
                    if split == "train":
                        id += 1
                        yield id_, {
                            "idx": id,
                            "premise": self._cleaner(row[0]),
                            "hypo": self._cleaner(row[1]),
                            "label": row[2].strip().replace("contradictory", "contradiction"),
                        }
                    else:
                        if row[0] == "fr":
                            id += 1
                            yield id_, {
                                "idx": id,
                                "premise": self._cleaner(row[6]),
                                "hypo": self._cleaner(row[7]),
                                "label": row[1].strip(),  # the label is already "contradiction" in the dev/test
                            }
        elif self.config.name == "WSD-V":
            wsd_rdr = WSDDatasetReader()
            for inst in wsd_rdr.read_from_data_dirs([os.path.join(data_file, split)]):
                yield inst[0], {
                    "idx": inst[0],
                    "sentence": inst[1],
                    "pos_tags": inst[2],
                    "lemmas": inst[3],
                    "fine_pos_tags": inst[4],
                    "disambiguate_tokens_ids": inst[5],
                    "disambiguate_labels": inst[6],
                }

    def _cls_extractor(self, line):
        """
        Extract review and label for CLS dataset
        from: https://github.com/getalp/Flaubert/blob/master/flue/extract_split_cls.py
        """
        m = re.search(r"(?<=<rating>)\d+.\d+(?=<\/rating>)", line)
        label = "positive" if int(float(m.group(0))) > 3 else "negative"  # rating == 3 are already removed
        category = re.search(r"(?<=<category>)\w+(?=<\/category>)", line)

        if category == "dvd":
            m = re.search(r"(?<=\/url><text>)(.|\n|\t|\f)+(?=\<\/title><summary>)", line)
        else:
            m = re.search(r"(?<=\/url><text>)(.|\n|\t|\f)+(?=\<\/text><title>)", line)

        review_text = m.group(0)

        return self._cleaner(review_text), label

    def _convert_to_unicode(self, text):
        """
        Converts `text` to Unicode (if it's not already), assuming UTF-8 input.
        from: https://github.com/getalp/Flaubert/blob/master/tools/clean_text.py
        """

        def ensure_text(s, encoding="utf-8", errors="strict"):
            if isinstance(s, bytes):
                return s.decode(encoding, errors)
            elif isinstance(s, str):
                return s
            else:
                raise TypeError("not expecting type '%s'" % type(s))

        return ensure_text(text, encoding="utf-8", errors="ignore")

    def _cleaner(self, text):
        """
        Clean up an input text
        from: https://github.com/getalp/Flaubert/blob/master/tools/clean_text.py
        """
        # Convert and normalize the unicode underlying representation
        text = self._convert_to_unicode(text)
        text = unicodedata.normalize("NFC", text)

        # Normalize whitespace characters and remove carriage return
        remap = {ord("\f"): " ", ord("\r"): "", ord("\n"): "", ord("\t"): ""}
        text = text.translate(remap)

        # Normalize URL links
        pattern = re.compile(r"(?:www|http)\S+|<\S+|\w+\/*>")
        text = re.sub(pattern, "", text)

        # remove multiple spaces in text
        pattern = re.compile(r"( ){2,}")
        text = re.sub(pattern, r" ", text)

        return text

    def _wsdv_prepare_data(self, dirpath):
        """Get data paths from FSE dir"""
        paths = {}

        for f in os.listdir(dirpath):
            if f.startswith("FSE"):
                data = "test"
            else:
                data = "train"

            paths["_".join((data, f))] = os.path.join(dirpath, f)

        test_dirpath = os.path.join(dirpath, "test")
        os.makedirs(test_dirpath, exist_ok=True)
        train_dirpath = os.path.join(dirpath, "train")
        os.makedirs(train_dirpath, exist_ok=True)
        # copy FSE file to new test directory
        for k, v in paths.items():
            data = k.split("_")[0]
            filename = k.split("_")[1]
            copyfile(v, os.path.join(dirpath, data, filename))


# The WSDDatasetReader classes come from https://github.com/getalp/Flaubert/blob/master/flue/wsd/verbs/modules/dataset.py
class WSDDatasetReader:
    """Class to read a WSD data directory. The directory should contain .data.xml and .gold.key.txt files"""

    def get_data_paths(self, indir):
        """Get file paths from WSD dir"""
        xml_fpath, gold_fpath = None, None

        for f in os.listdir(indir):
            if f.endswith(".data.xml"):
                xml_fpath = os.path.join(indir, f)
            if f.endswith(".gold.key.txt"):
                gold_fpath = os.path.join(indir, f)
        return xml_fpath, gold_fpath

    def read_gold(self, infile):
        """Read .gold.key.txt and return data as dict.
        :param infile: fpath to .gold.key.txt file
        :type infile: str
        :return: return data into dict format : {str(instance_id): set(label)}
        :rtype: dict
        """
        return {
            line.split()[0]: tuple(line.rstrip("\n").split()[1:])
            for line in open(infile, encoding="utf-8").readlines()
        }

    def read_from_data_dirs(self, data_dirs):
        """Read WSD data and return as WSDDataset"""
        for d in data_dirs:
            xml_fpath, gold_fpath = self.get_data_paths(d)

            # read gold file
            id2gold = self.read_gold(gold_fpath)

            sentences = self.read_sentences(d)

            # Parse xml
            tree = etree.parse(xml_fpath)
            corpus = tree.getroot()

            # process data
            # iterate over document
            for text in corpus:
                # iterates over sentences
                for sentence in text:
                    sent_id = sentence.get("id")  # sentence id
                    sent = next(sentences)  # get sentence
                    pos_tags = []
                    lemmas = []
                    fine_pos_tags = []
                    disambiguate_tokens_ids = []
                    disambiguate_labels = []
                    tok_idx = 0

                    # iterate over tokens
                    for tok in sentence:
                        lemma, pos, fine_pos_tag = tok.get("lemma"), tok.get("pos"), tok.get("fine_pos")

                        pos_tags.append(pos)
                        lemmas.append(lemma)
                        fine_pos_tags.append(fine_pos_tag)
                        wf = tok.text
                        subtokens = wf.split(" ")

                        # add sense annotated token
                        if tok.tag == "instance":
                            id = tok.get("id")

                            target_labels = id2gold[id]
                            target_first_label = target_labels[0]

                            # We focus on the head of the target mwe instance
                            if pos == "VERB":
                                tgt_idx = tok_idx  # head is mostly the first token as most mwe verb targets are phrasal verbs (i.g lift up)
                            else:
                                tgt_idx = (
                                    tok_idx + len(subtokens) - 1
                                )  # other pos head are generally the last token of the mwe (i.g European Union)

                            disambiguate_tokens_ids.append(tgt_idx)
                            disambiguate_labels.append(target_first_label)

                        tok_idx += 1

                    yield (
                        sent_id,
                        sent,
                        pos_tags,
                        lemmas,
                        fine_pos_tags,
                        disambiguate_tokens_ids,
                        disambiguate_labels,
                    )

    def read_sentences(self, data_dir, keep_mwe=True):
        """Read sentences from WSD data"""

        xml_fpath, _ = self.get_data_paths(data_dir)
        return self.read_sentences_from_xml(xml_fpath, keep_mwe=keep_mwe)

    def read_sentences_from_xml(self, infile, keep_mwe=False):
        """Read sentences from xml file"""

        # Parse xml
        tree = etree.parse(infile)
        corpus = tree.getroot()

        for text in corpus:
            for sentence in text:
                if keep_mwe:
                    sent = [tok.text.replace(" ", "_") for tok in sentence]
                else:
                    sent = [subtok for tok in sentence for subtok in tok.text.split(" ")]
                yield sent

    def read_target_keys(self, infile):
        """Read target keys"""
        return [x.rstrip("\n") for x in open(infile, encoding="utf-8").readlines()]



"""TODO(lc_quad): Add a description here."""


import json
import os

import datasets


# TODO(lc_quad): BibTeX citation
_CITATION = """
@inproceedings{dubey2017lc2,
title={LC-QuAD 2.0: A Large Dataset for Complex Question Answering over Wikidata and DBpedia},
author={Dubey, Mohnish and Banerjee, Debayan and Abdelkawi, Abdelrahman and Lehmann, Jens},
booktitle={Proceedings of the 18th International Semantic Web Conference (ISWC)},
year={2019},
organization={Springer}
}
"""

# TODO(lc_quad):
_DESCRIPTION = """\
LC-QuAD 2.0 is a Large Question Answering dataset with 30,000 pairs of question and its corresponding SPARQL query. The target knowledge base is Wikidata and DBpedia, specifically the 2018 version. Please see our paper for details about the dataset creation process and framework.
"""
_URL = "https://github.com/AskNowQA/LC-QuAD2.0/archive/master.zip"


class LcQuad(datasets.GeneratorBasedBuilder):
    """TODO(lc_quad): Short description of my dataset."""

    # TODO(lc_quad): Set up version.
    VERSION = datasets.Version("2.0.0")

    def _info(self):
        # TODO(lc_quad): Specifies the datasets.DatasetInfo object
        return datasets.DatasetInfo(
            # This is the description that will appear on the datasets page.
            description=_DESCRIPTION,
            # datasets.features.FeatureConnectors
            features=datasets.Features(
                {
                    "NNQT_question": datasets.Value("string"),
                    "uid": datasets.Value("int32"),
                    "subgraph": datasets.Value("string"),
                    "template_index": datasets.Value("int32"),
                    "question": datasets.Value("string"),
                    "sparql_wikidata": datasets.Value("string"),
                    "sparql_dbpedia18": datasets.Value("string"),
                    "template": datasets.Value("string"),
                    # "template_id": datasets.Value('string'),
                    "paraphrased_question": datasets.Value("string")
                    # These are the features of your dataset like images, labels ...
                }
            ),
            # If there's a common (input, target) tuple from the features,
            # specify them here. They'll be used if as_supervised=True in
            # builder.as_dataset.
            supervised_keys=None,
            # Homepage of the dataset for documentation
            homepage="http://lc-quad.sda.tech/",
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""
        # TODO(lc_quad): Downloads the data and defines the splits
        # dl_manager is a datasets.download.DownloadManager that can be used to
        # download and extract URLs
        dl_dir = dl_manager.download_and_extract(_URL)
        dl_dir = os.path.join(dl_dir, "LC-QuAD2.0-master", "dataset")
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                # These kwargs will be passed to _generate_examples
                gen_kwargs={"filepath": os.path.join(dl_dir, "train.json")},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                # These kwargs will be passed to _generate_examples
                gen_kwargs={"filepath": os.path.join(dl_dir, "test.json")},
            ),
        ]

    def _generate_examples(self, filepath):
        """Yields examples."""
        # TODO(lc_quad): Yields (key, example) tuples from the dataset
        with open(filepath, encoding="utf-8") as f:
            data = json.load(f)
            for id_, row in enumerate(data):
                is_list = False
                for key in row:
                    if key != "answer" and isinstance(row[key], list):
                        is_list = True
                if is_list:
                    continue
                yield id_, {
                    "NNQT_question": row["NNQT_question"],
                    "uid": row["uid"],
                    "subgraph": row["subgraph"],
                    "template_index": row["template_index"],
                    "question": row["question"],
                    "sparql_wikidata": row["sparql_wikidata"],
                    "sparql_dbpedia18": row["sparql_dbpedia18"],
                    "template": row["template"],
                    # "template_id": str(row['template_id']),
                    "paraphrased_question": row["paraphrased_question"],
                }



# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Inferring Which Medical Treatments Work from Reports of Clinical Trials"""


import csv
import os

import datasets


_CITATION = """\
@inproceedings{lehman-etal-2019-inferring,
    title = "Inferring Which Medical Treatments Work from Reports of Clinical Trials",
    author = "Lehman, Eric  and
      DeYoung, Jay  and
      Barzilay, Regina  and
      Wallace, Byron C.",
    booktitle = "Proceedings of the 2019 Conference of the North {A}merican Chapter of the Association for Computational Linguistics: Human Language Technologies, Volume 1 (Long and Short Papers)",
    month = jun,
    year = "2019",
    address = "Minneapolis, Minnesota",
    publisher = "Association for Computational Linguistics",
    url = "https://www.aclweb.org/anthology/N19-1371",
    pages = "3705--3717",
}
"""

_DESCRIPTION = """\
Data and code from our "Inferring Which Medical Treatments Work from Reports of Clinical Trials", NAACL 2019. This work concerns inferring the results reported in clinical trials from text.

The dataset consists of biomedical articles describing randomized control trials (RCTs) that compare multiple treatments. Each of these articles will have multiple questions, or 'prompts' associated with them. These prompts will ask about the relationship between an intervention and comparator with respect to an outcome, as reported in the trial. For example, a prompt may ask about the reported effects of aspirin as compared to placebo on the duration of headaches. For the sake of this task, we assume that a particular article will report that the intervention of interest either significantly increased, significantly decreased or had significant effect on the outcome, relative to the comparator.

The dataset could be used for automatic data extraction of the results of a given RCT. This would enable readers to discover the effectiveness of different treatments without needing to read the paper.
"""


class EvidenceInferenceConfig(datasets.BuilderConfig):
    """BuilderConfig for NewDataset"""

    def __init__(self, zip_file, **kwargs):
        """

        Args:
            zip_file: The location of zip file containing original data
            **kwargs: keyword arguments forwarded to super.
        """
        self.zip_file = zip_file
        super().__init__(**kwargs)


class EvidenceInferTreatment(datasets.GeneratorBasedBuilder):
    f"""{_DESCRIPTION}"""

    # This is an example of a dataset with multiple configurations.
    # If you don't want/need to define several sub-sets in your dataset,
    # just remove the BUILDER_CONFIG_CLASS and the BUILDER_CONFIGS attributes.
    BUILDER_CONFIG_CLASS = EvidenceInferenceConfig
    BUILDER_CONFIGS = [
        EvidenceInferenceConfig(
            name="2.0",
            description="EvidenceInference V2",
            version=datasets.Version("2.0.0"),
            zip_file="https://github.com/jayded/evidence-inference/archive/refs/tags/v2.0.zip",
        ),
        EvidenceInferenceConfig(
            name="1.1",
            description="EvidenceInference V1.1",
            version=datasets.Version("1.1.0"),
            zip_file="https://github.com/jayded/evidence-inference/archive/v1.1.zip",
        ),
    ]

    def _info(self):
        features = datasets.Features(
            {
                "Text": datasets.Value("string"),
                "PMCID": datasets.Value("int32"),
                "Prompts": datasets.Sequence(
                    datasets.Features(
                        {
                            "PromptID": datasets.Value("int32"),
                            "PMCID": datasets.Value("int32"),
                            "Outcome": datasets.Value("string"),
                            "Intervention": datasets.Value("string"),
                            "Comparator": datasets.Value("string"),
                            "Annotations": datasets.Sequence(
                                datasets.Features(
                                    {
                                        "UserID": datasets.Value("int32"),
                                        "PromptID": datasets.Value("int32"),
                                        "PMCID": datasets.Value("int32"),
                                        "Valid Label": datasets.Value("bool"),
                                        "Valid Reasoning": datasets.Value("bool"),
                                        "Label": datasets.Value("string"),
                                        "Annotations": datasets.Value("string"),
                                        "Label Code": datasets.Value("int32"),
                                        "In Abstract": datasets.Value("bool"),
                                        "Evidence Start": datasets.Value("int32"),
                                        "Evidence End": datasets.Value("int32"),
                                    }
                                )
                            ),
                        }
                    )
                ),
            }
        )

        return datasets.DatasetInfo(
            # This is the description that will appear on the datasets page.
            description=_DESCRIPTION,
            # datasets.features.FeatureConnectors
            features=features,
            # If there's a common (input, target) tuple from the features,
            # specify them here. They'll be used if as_supervised=True in
            # builder.as_dataset.
            supervised_keys=None,
            # Homepage of the dataset for documentation
            homepage="https://github.com/jayded/evidence-inference",
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        dl_dir = dl_manager.download_and_extract(self.config.zip_file)
        dl_dir = os.path.join(dl_dir, f"evidence-inference-{self.config.name}", "annotations")

        SPLITS = {}
        for split in ["train", "test", "validation"]:
            filename = os.path.join(dl_dir, "splits", f"{split}_article_ids.txt")
            with open(filename, "r", encoding="utf-8") as f:
                for line in f:
                    id_ = int(line.strip())
                    SPLITS[id_] = split

        ALL_PROMPTS = {}
        prompts_filename = os.path.join(dl_dir, "prompts_merged.csv")
        with open(prompts_filename, "r", encoding="utf-8") as f:
            data = csv.DictReader(f)
            for item in data:
                prompt_id = int(item["PromptID"])
                ALL_PROMPTS[prompt_id] = {"Prompt": item, "Annotations": []}

        annotations_filename = os.path.join(dl_dir, "annotations_merged.csv")
        with open(annotations_filename, "r", encoding="utf-8") as f:
            data = csv.DictReader(f)
            for item in data:
                prompt_id = int(item["PromptID"])

                if "Annotations" not in ALL_PROMPTS[prompt_id]:
                    ALL_PROMPTS[prompt_id]["Annotations"] = []

                ALL_PROMPTS[prompt_id]["Annotations"].append(item)

        # Simplify everything
        directory = os.path.join(dl_dir, "txt_files")
        ALL_IDS = {"train": [], "test": [], "validation": []}
        for prompt_id, item in ALL_PROMPTS.items():
            pmcid = int(item["Prompt"]["PMCID"])
            if pmcid not in SPLITS:
                if os.path.isfile(os.path.join(directory, f"PMC{pmcid}.txt")):
                    split = "train"
                else:
                    continue
            else:
                split = SPLITS[pmcid]

            values = ALL_IDS[split]

            filtered = [v for v in values if v["PMCID"] == pmcid]
            if len(filtered) == 1:
                value = filtered[0]
            else:
                value = {"PMCID": pmcid, "Prompts": []}
                values.append(value)

            new_item = item["Prompt"]
            new_item["Annotations"] = item["Annotations"]
            value["Prompts"].append(new_item)

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                # These kwargs will be passed to _generate_examples
                gen_kwargs={
                    "directory": os.path.join(dl_dir, "txt_files"),
                    "items": ALL_IDS["train"],
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                # These kwargs will be passed to _generate_examples
                gen_kwargs={
                    "directory": directory,
                    "items": ALL_IDS["test"],
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                # These kwargs will be passed to _generate_examples
                gen_kwargs={
                    "directory": os.path.join(dl_dir, "txt_files"),
                    "items": ALL_IDS["validation"],
                },
            ),
        ]

    def _generate_examples(self, directory, items):
        """Yields examples."""
        for id_, item in enumerate(items):
            pmcid = item["PMCID"]
            filename = os.path.join(directory, f"PMC{pmcid}.txt")
            with open(filename, "r", encoding="utf-8") as f:
                text = f.read()

            yield id_, {"Text": text, **item}



# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""The Winograd Schema Challenge Dataset"""

import xml.etree.ElementTree as ET

import datasets


_DESCRIPTION = """\
A Winograd schema is a pair of sentences that differ in only one or two words and that contain an ambiguity that is
resolved in opposite ways in the two sentences and requires the use of world knowledge and reasoning for its
resolution. The schema takes its name from a well-known example by Terry Winograd:

> The city councilmen refused the demonstrators a permit because they [feared/advocated] violence.

If the word is ``feared'', then ``they'' presumably refers to the city council; if it is ``advocated'' then ``they''
presumably refers to the demonstrators.
"""

_CITATION = """\
@inproceedings{levesque2012winograd,
  title={The winograd schema challenge},
  author={Levesque, Hector and Davis, Ernest and Morgenstern, Leora},
  booktitle={Thirteenth International Conference on the Principles of Knowledge Representation and Reasoning},
  year={2012},
  organization={Citeseer}
}
"""

_HOMPAGE = "https://cs.nyu.edu/faculty/davise/papers/WinogradSchemas/WS.html"
_DOWNLOAD_URL = "https://cs.nyu.edu/faculty/davise/papers/WinogradSchemas/WSCollection.xml"


class WinogradWSCConfig(datasets.BuilderConfig):
    """BuilderConfig for WinogradWSC."""

    def __init__(self, *args, language=None, inds=None, **kwargs):
        super().__init__(*args, **kwargs)
        self.inds = set(inds) if inds is not None else None

    def is_in_range(self, id):
        """Takes an index and tells you if it belongs to the configuration's subset"""
        return id in self.inds if self.inds is not None else True


class WinogradWSC(datasets.GeneratorBasedBuilder):
    """The Winograd Schema Challenge Dataset"""

    BUILDER_CONFIG_CLASS = WinogradWSCConfig
    BUILDER_CONFIGS = [
        WinogradWSCConfig(
            name="wsc285",
            description="Full set of winograd examples",
        ),
        WinogradWSCConfig(
            name="wsc273",
            description="A commonly-used subset of examples. Identical to 'wsc285' but without the last 12 examples.",
            inds=list(range(273)),
        ),
    ]

    def _info(self):
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(
                {
                    "text": datasets.Value("string"),
                    "pronoun": datasets.Value("string"),
                    "pronoun_loc": datasets.Value("int32"),
                    "quote": datasets.Value("string"),
                    "quote_loc": datasets.Value("int32"),
                    "options": datasets.Sequence(datasets.Value("string")),
                    "label": datasets.ClassLabel(num_classes=2),
                    "source": datasets.Value("string"),
                }
            ),
            homepage=_HOMPAGE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        path = dl_manager.download_and_extract(_DOWNLOAD_URL)
        return [
            datasets.SplitGenerator(name=datasets.Split.TEST, gen_kwargs={"filepath": path}),
        ]

    def _cleanup_whitespace(self, text):
        return " ".join(text.split())

    def _generate_examples(self, filepath):
        tree = ET.parse(filepath)
        for id, schema in enumerate(tree.getroot()):
            if not self.config.is_in_range(id):
                continue

            text_root = schema.find("text")
            quote_root = schema.find("quote")

            text_left = self._cleanup_whitespace(text_root.findtext("txt1", ""))
            text_right = self._cleanup_whitespace(text_root.findtext("txt2", ""))
            quote_left = self._cleanup_whitespace(quote_root.findtext("quote1", ""))
            quote_right = self._cleanup_whitespace(quote_root.findtext("quote2", ""))
            pronoun = self._cleanup_whitespace(text_root.findtext("pron"))

            features = {}
            features["text"] = " ".join([text_left, pronoun, text_right]).strip()
            features["quote"] = " ".join([quote_left, pronoun, quote_right]).strip()

            features["pronoun"] = pronoun
            features["options"] = [
                self._cleanup_whitespace(option.text) for option in schema.find("answers").findall("answer")
            ]

            answer_txt = self._cleanup_whitespace(schema.findtext("correctAnswer"))
            features["label"] = int("B" in answer_txt)  # convert "  A. " or " B " strings to a 0/1 index

            features["pronoun_loc"] = len(text_left) + 1 if len(text_left) > 0 else 0
            features["quote_loc"] = features["pronoun_loc"] - (len(quote_left) + 1 if len(quote_left) > 0 else 0)
            features["source"] = self._cleanup_whitespace(schema.findtext("source"))

            yield id, features



# coding=utf-8
# Copyright 2020 HuggingFace Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Lint as: python3
"""Named entities Recognition dataset for Norwegian."""

import conllu

import datasets


_CITATION = """\
@inproceedings{johansen2019ner,
  title={Named-Entity Recognition for Norwegian},
  author={Johansen, Bjarte},
  booktitle={Proceedings of the 22nd Nordic Conference on Computational Linguistics, NoDaLiDa},
  year={2019}
}
"""

_DESCRIPTION = """\
Named entities Recognition dataset for Norwegian. It is
a version of the Universal Dependency (UD) Treebank for both Bokmål and Nynorsk (UDN) where
all proper nouns have been tagged with their type according to the NER tagging scheme. UDN is a converted
version of the Norwegian Dependency Treebank into the UD scheme.
"""

# Files are stored with Git LFS then we add raw=true at the end
_URL = "https://github.com/ljos/navnkjenner/blob/master/data/"
_BOKMAAL_TRAIN = "bokmaal/no_bokmaal-ud-train.bioes?raw=true"
_BOKMAAL_DEV = "bokmaal/no_bokmaal-ud-dev.bioes?raw=true"
_BOKMAAL_TEST = "bokmaal/no_bokmaal-ud-test.bioes?raw=true"
_NYNORSK_TRAIN = "nynorsk/no_nynorsk-ud-train.bioes?raw=true"
_NYNORSK_DEV = "nynorsk/no_nynorsk-ud-dev.bioes?raw=true"
_NYNORSK_TEST = "nynorsk/no_nynorsk-ud-test.bioes?raw=true"
_SAMNORSK_TRAIN = "samnorsk/no_samnorsk-ud-train.bioes?raw=true"
_SAMNORSK_DEV = "samnorsk/no_samnorsk-ud-dev.bioes?raw=true"
_SAMNORSK_TEST = "samnorsk/no_samnorsk-ud-test.bioes?raw=true"


class NorwegiannerConfig(datasets.BuilderConfig):
    """BuilderConfig for NorwegianNER."""

    def __init__(self, **kwargs):
        """BuilderConfig for Norwegianner.

        Args:
          **kwargs: keyword arguments forwarded to super.
        """
        super(NorwegiannerConfig, self).__init__(**kwargs)


class Norwegianner(datasets.GeneratorBasedBuilder):
    """Norwegianner dataset."""

    BUILDER_CONFIGS = [
        NorwegiannerConfig(
            name="bokmaal", version=datasets.Version("1.0.0"), description="Norwegianner bokmaal dataset"
        ),
        NorwegiannerConfig(
            name="nynorsk", version=datasets.Version("1.0.0"), description="Norwegianner nynorsk dataset"
        ),
        NorwegiannerConfig(
            name="samnorsk", version=datasets.Version("1.0.0"), description="Norwegianner samnorsk dataset"
        ),
    ]

    def _info(self):
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(
                {
                    "idx": datasets.Value("string"),
                    "text": datasets.Value("string"),
                    "tokens": datasets.Sequence(datasets.Value("string")),
                    "lemmas": datasets.Sequence(datasets.Value("string")),
                    "pos_tags": datasets.Sequence(
                        datasets.features.ClassLabel(
                            names=[
                                "NOUN",
                                "PUNCT",
                                "ADP",
                                "NUM",
                                "SYM",
                                "SCONJ",
                                "ADJ",
                                "PART",
                                "DET",
                                "CCONJ",
                                "PROPN",
                                "PRON",
                                "X",
                                "ADV",
                                "INTJ",
                                "VERB",
                                "AUX",
                            ]
                        )
                    ),
                    "ner_tags": datasets.Sequence(
                        datasets.features.ClassLabel(
                            names=[
                                "O",
                                "B-OTH",
                                "I-OTH",
                                "E-OTH",
                                "S-OTH",
                                "B-ORG",
                                "I-ORG",
                                "E-ORG",
                                "S-ORG",
                                "B-PRS",
                                "I-PRS",
                                "E-PRS",
                                "S-PRS",
                                "B-GEO",
                                "I-GEO",
                                "E-GEO",
                                "S-GEO",
                            ]
                        )
                    ),
                }
            ),
            supervised_keys=None,
            homepage="https://github.com/ljos/navnkjenner",
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""
        if self.config.name == "bokmaal":
            urls_to_download = {
                "train": f"{_URL}{_BOKMAAL_TRAIN}",
                "dev": f"{_URL}{_BOKMAAL_DEV}",
                "test": f"{_URL}{_BOKMAAL_TEST}",
            }
        elif self.config.name == "nynorsk":
            urls_to_download = {
                "train": f"{_URL}{_NYNORSK_TRAIN}",
                "dev": f"{_URL}{_NYNORSK_DEV}",
                "test": f"{_URL}{_NYNORSK_TEST}",
            }
        elif self.config.name == "samnorsk":
            urls_to_download = {
                "train": f"{_URL}{_SAMNORSK_TRAIN}",
                "dev": f"{_URL}{_SAMNORSK_DEV}",
                "test": f"{_URL}{_SAMNORSK_TEST}",
            }

        downloaded_files = dl_manager.download_and_extract(urls_to_download)

        return [
            datasets.SplitGenerator(name=datasets.Split.TRAIN, gen_kwargs={"filepath": downloaded_files["train"]}),
            datasets.SplitGenerator(name=datasets.Split.VALIDATION, gen_kwargs={"filepath": downloaded_files["dev"]}),
            datasets.SplitGenerator(name=datasets.Split.TEST, gen_kwargs={"filepath": downloaded_files["test"]}),
        ]

    def _generate_examples(self, filepath):
        with open(filepath, "r", encoding="utf-8") as data_file:
            tokenlist = list(conllu.parse_incr(data_file))
            id = 0
            for sent in tokenlist:
                yield id, {
                    "idx": sent.metadata["sent_id"],
                    "text": sent.metadata["text"],
                    "tokens": [token["form"] for token in sent],
                    "lemmas": [token["lemma"] for token in sent],
                    "pos_tags": [token["upos"] for token in sent],
                    "ner_tags": [token["xpos"] for token in sent],
                }
                id += 1



# coding=utf-8
# Copyright 2020 HuggingFace Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Lint as: python3

import datasets


_DESCRIPTION = """\
This dataset was curated from the Bing search logs (desktop users only) over the period of Jan 1st, 2020 – (Current Month - 1). Only searches that were issued many times by multiple users were included. The dataset includes queries from all over the world that had an intent related to the Coronavirus or Covid-19. In some cases this intent is explicit in the query itself (e.g., “Coronavirus updates Seattle”), in other cases it is implicit , e.g. “Shelter in place”. The implicit intent of search queries (e.g., “Toilet paper”) was extracted using random walks on the click graph as outlined in this paper by Microsoft Research. All personal data were removed.
"""
_HOMEPAGE_URL = "https://github.com/microsoft/BingCoronavirusQuerySet"
_CITATION = None

_VERSION = "1.0.0"
_BASE_URL = "https://raw.githubusercontent.com/microsoft/BingCoronavirusQuerySet/master/data/2020/{}_{}_{}.tsv"


class BingCoronavirusQuerySetConfig(datasets.BuilderConfig):
    def __init__(self, *args, queries_by=None, start_date=None, end_date=None, **kwargs):
        super().__init__(
            *args,
            name=f"{queries_by}_{start_date}_{end_date}",
            **kwargs,
        )
        self.queries_by = queries_by
        self.start_date = start_date
        self.end_date = end_date


class BingCoronavirusQuerySet(datasets.GeneratorBasedBuilder):
    BUILDER_CONFIGS = [
        BingCoronavirusQuerySetConfig(
            queries_by="country",
            start_date="2020-09-01",
            end_date="2020-09-30",
            description="Query by: country, start_date: 2020-09-01, end_date: 2020-09-30",
            version=datasets.Version(_VERSION),
        )
    ]
    BUILDER_CONFIG_CLASS = BingCoronavirusQuerySetConfig

    def _info(self):
        if self.config.queries_by == "country":
            features = datasets.Features(
                {
                    "id": datasets.Value("int32"),
                    "Date": datasets.Value("string"),
                    "Query": datasets.Value("string"),
                    "IsImplicitIntent": datasets.Value("string"),
                    "Country": datasets.Value("string"),
                    "PopularityScore": datasets.Value("int32"),
                },
            )
        elif self.config.queries_by == "state":
            features = datasets.Features(
                {
                    "id": datasets.Value("int32"),
                    "Date": datasets.Value("string"),
                    "Query": datasets.Value("string"),
                    "IsImplicitIntent": datasets.Value("string"),
                    "State": datasets.Value("string"),
                    "Country": datasets.Value("string"),
                    "PopularityScore": datasets.Value("int32"),
                },
            )
        else:
            raise Exception("Unknown queries_by. Choose one of: country or state")
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            supervised_keys=None,
            homepage=_HOMEPAGE_URL,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        def _base_url(queries_by, start_date, end_date):
            if queries_by == "country":
                queries_by = "QueriesByCountry"
            else:
                queries_by = "QueriesByState"
            return _BASE_URL.format(queries_by, start_date, end_date)

        download_url = _base_url(self.config.queries_by, self.config.start_date, self.config.end_date)
        path = dl_manager.download_and_extract(download_url)
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={"datapath": path},
            )
        ]

    def _generate_examples(self, datapath):
        with open(datapath, encoding="utf-8") as f:
            for sentence_counter, row in enumerate(f):
                if sentence_counter == 0:
                    continue
                row = row.strip().split("\t")
                if self.config.queries_by == "country":
                    resp = {
                        "id": sentence_counter,
                        "Date": row[0],
                        "Query": row[1],
                        "IsImplicitIntent": row[2],
                        "Country": row[3],
                        "PopularityScore": row[4],
                    }
                else:
                    resp = {
                        "id": sentence_counter,
                        "Date": row[0],
                        "Query": row[1],
                        "IsImplicitIntent": row[2],
                        "State": row[3],
                        "Country": row[4],
                        "PopularityScore": int(row[5]),
                    }
                yield sentence_counter, resp



# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""TODO: Add a description here."""


import json
import os

import datasets


_CITATION = """\
@article{hardalov2019beyond,
  title={Beyond english-only reading comprehension: Experiments in zero-shot multilingual transfer for bulgarian},
  author={Hardalov, Momchil and Koychev, Ivan and Nakov, Preslav},
  journal={arXiv preprint arXiv:1908.01519},
  year={2019}
}
"""

_DESCRIPTION = """\
This new dataset is designed to do reading comprehension in Bulgarian language.
"""

_HOMEPAGE = "https://github.com/mhardalov/bg-reason-BERT"

_LICENSE = "Apache-2.0 License"

_URL = "https://raw.githubusercontent.com/mhardalov/bg-reason-BERT/master/data/bg_rc-v1.0.json"


class ReasoningBg(datasets.GeneratorBasedBuilder):
    """To test reasoning comprehension in Bulgarian Language"""

    VERSION = datasets.Version("1.1.0")

    BUILDER_CONFIGS = [
        datasets.BuilderConfig(
            name="biology-12th", version=VERSION, description="MCQs in Biology of 12th Grade Matriculation Exam"
        ),
        datasets.BuilderConfig(
            name="philosophy-12th", version=VERSION, description="MCQs in Philosophy of 12th Grade Matriculation Exam"
        ),
        datasets.BuilderConfig(
            name="geography-12th", version=VERSION, description="MCQs in Geography of 12th Grade Matriculation Exam"
        ),
        datasets.BuilderConfig(
            name="history-12th", version=VERSION, description="MCQs in History of 12th Grade Matriculation Exam"
        ),
        datasets.BuilderConfig(
            name="history-quiz", version=VERSION, description="MCQs in Bulgarian History from Online History Quizzes"
        ),
    ]

    def _info(self):
        features = datasets.Features(
            {
                "id": datasets.Value("string"),
                "url": datasets.Value("string"),
                "qid": datasets.Value("int32"),
                "question": datasets.Value("string"),
                "answers": datasets.Sequence(datasets.Value("string")),
                "correct": datasets.Value("string"),
            }
        )
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            supervised_keys=None,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""

        downloaded_path = dl_manager.download(_URL)
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                # These kwargs will be passed to _generate_examples
                gen_kwargs={
                    "filepath": os.path.join(downloaded_path),
                    "split": "train",
                },
            ),
        ]

    def _generate_examples(self, filepath, split):
        """Yields examples."""

        with open(filepath, encoding="utf-8") as f:
            content = f.read()
            data = json.loads(content)["data"][self.config.name]
            for page in data:
                url = page["url"]
                for question in page["questions"]:
                    yield str(question["id"]), {
                        "url": url,
                        "id": question["id"],
                        "qid": question["qid"],
                        "question": question["question"],
                        "answers": question["answers"],
                        "correct": question["correct"],
                    }



"""Public domain texts from Project Ben-Yehuda- a set of books extracted from the Project BenYehuda library"""


import csv

import datasets


_CITATION = """\
@article{,
  author = {},
  title = {Public domain texts from Project Ben-Yehuda},
  journal = {},
  url = {https://github.com/projectbenyehuda/public_domain_dump},
  year = {2020},
}

"""

_DESCRIPTION = """\
This repository contains a dump of thousands of public domain works in Hebrew, from Project Ben-Yehuda, in plaintext UTF-8 files, with and without diacritics (nikkud). The metadata (pseudocatalogue.csv) file is a list of titles, authors, genres, and file paths, to help you process the dump.
All these works are in the public domain, so you are free to make any use of them, and do not need to ask for permission.
There are 10078 files, 3181136 lines
"""

_ASSET_ROOT_URL = "https://raw.githubusercontent.com/projectbenyehuda/public_domain_dump/master/"
_STORAGE_API_ROOT_URL = "https://raw.githubusercontent.com/projectbenyehuda/public_domain_dump/master/txt/"

# download one by one file from github is too slow

_METADATA_URL = _ASSET_ROOT_URL + "pseudocatalogue.csv"


class HebrewProjectbenyehuda(datasets.GeneratorBasedBuilder):
    """Project Ben Yehuda dataset - books as plain text extracted from the Project Project Ben Yehuda library"""

    VERSION = datasets.Version("0.1.0")

    def _info(self):
        return datasets.DatasetInfo(
            # This is the description that will appear on the datasets page.
            description=_DESCRIPTION,
            # datasets.features.FeatureConnectors
            features=datasets.Features(
                {
                    "id": datasets.Value("int32"),
                    "url": datasets.Value("string"),
                    "title": datasets.Value("string"),
                    "authors": datasets.Value("string"),
                    "translators": datasets.Value("string"),
                    "original_language": datasets.Value("string"),
                    "genre": datasets.Value("string"),
                    "source_edition": datasets.Value("string"),
                    "text": datasets.Value("string"),
                    # These are the features of your dataset like images, labels ...
                }
            ),
            # If there's a common (input, target) tuple from the features,
            # specify them here. They'll be used if as_supervised=True in
            # builder.as_dataset.
            supervised_keys=None,
            # Homepage of the dataset for documentation
            homepage="https://github.com/projectbenyehuda/public_domain_dump",
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""

        # dl_manager is a datasets.download.DownloadManager that can be used to
        # download and extract URLs

        metadata = dl_manager.download({"metadata": _METADATA_URL})

        urls_to_download = dict()
        ids = list()
        with open(metadata["metadata"], encoding="utf-8") as csv_file:
            for row in csv.DictReader(csv_file):
                ids.append(row["ID"])
                urls_to_download[row["ID"]] = _STORAGE_API_ROOT_URL + row["path"].strip("/") + ".txt"

        downloaded_files = dl_manager.download(urls_to_download)
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "ids": ids,
                    "metadata_filepath": metadata["metadata"],
                    "filepaths": downloaded_files,
                },
            )
        ]

    def _generate_examples(self, ids, metadata_filepath, filepaths):
        """Yields examples."""

        with open(metadata_filepath, encoding="utf-8") as f:
            metadata_dict = csv.DictReader(
                f,
                fieldnames=[
                    "_id",
                    "path",
                    "title",
                    "authors",
                    "translators",
                    "original_language",
                    "genre",
                    "source_edition",
                ],
            )
            indexed_metadata = {str(row["_id"]): row for row in metadata_dict}

        for _id in ids:
            data = indexed_metadata[_id]
            filepath = filepaths[_id]

            with open(filepath, encoding="utf-8") as f:
                text = f.read()

            _id = data["_id"]
            title = data["title"]
            url = data["path"].strip("/")
            url = _STORAGE_API_ROOT_URL + url + ".txt"
            authors = data["authors"]
            translators = data["translators"]
            original_language = data["original_language"]
            genre = data["genre"]
            source_edition = data["source_edition"]

            yield _id, {
                "id": _id,
                "title": title,
                "url": url,
                "authors": authors,
                "translators": translators,
                "original_language": original_language,
                "genre": genre,
                "source_edition": source_edition,
                "text": text,
            }



# coding=utf-8
# Copyright 2020 The TensorFlow Datasets Authors and the HuggingFace Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import csv

import datasets


_CITATION = """\
@inproceedings{hasanain2016questions,
  title={What Questions Do Journalists Ask on Twitter?},
  author={Hasanain, Maram and Bagdouri, Mossaab and Elsayed, Tamer and Oard, Douglas W},
  booktitle={Tenth International AAAI Conference on Web and Social Media},
  year={2016}
}
"""

_DESCRIPTION = """\
The journalists_questions corpus (version 1.0) is a collection of 10K human-written Arabic
tweets manually labeled for question identification over Arabic tweets posted by journalists.
"""
_DATA_URL = "https://drive.google.com/uc?export=download&id=1CBrh-9OrSpKmPQBxTK_ji6mq6WTN_U9U"


class JournalistsQuestions(datasets.GeneratorBasedBuilder):
    BUILDER_CONFIGS = [
        datasets.BuilderConfig(
            name="plain_text",
            version=datasets.Version("1.0.0", ""),
            description="Journalists tweet IDs and annotation by whether the tweet has a question",
        )
    ]

    def _info(self):
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(
                {
                    "tweet_id": datasets.Value("string"),
                    "label": datasets.features.ClassLabel(names=["no", "yes"]),
                    "label_confidence": datasets.Value("float"),
                }
            ),
            homepage="http://qufaculty.qu.edu.qa/telsayed/datasets/",
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        dl_dir = dl_manager.download_and_extract(_DATA_URL)
        return [
            datasets.SplitGenerator(name=datasets.Split.TRAIN, gen_kwargs={"filepath": dl_dir}),
        ]

    def _generate_examples(self, filepath):
        """This function returns the examples in the raw (text) form."""
        with open(filepath, encoding="utf-8") as f:
            reader = csv.DictReader(f, delimiter="\t", fieldnames=["tweet_id", "label", "label_confidence"])
            for idx, row in enumerate(reader):
                yield idx, {
                    "tweet_id": row["tweet_id"],
                    "label": row["label"],
                    "label_confidence": float(row["label_confidence"]),
                }



# coding=utf-8
# Copyright 2020 The TensorFlow Datasets Authors and the HuggingFace Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Lint as: python3
"""SMS Spam Collection Data Set"""


import os

import datasets
from datasets.tasks import TextClassification


_CITATION = """\
@inproceedings{Almeida2011SpamFiltering,
  title={Contributions to the Study of SMS Spam Filtering: New Collection and Results},
  author={Tiago A. Almeida and Jose Maria Gomez Hidalgo and Akebo Yamakami},
  year={2011},
  booktitle = "Proceedings of the 2011 ACM Symposium on Document Engineering (DOCENG'11)",
}
"""

_DESCRIPTION = """\
The SMS Spam Collection v.1 is a public set of SMS labeled messages that have been collected for mobile phone spam research.
It has one collection composed by 5,574 English, real and non-enconded messages, tagged according being legitimate (ham) or spam.
"""

_DATA_URL = "http://archive.ics.uci.edu/ml/machine-learning-databases/00228/smsspamcollection.zip"


class SmsSpam(datasets.GeneratorBasedBuilder):
    """SMS Spam Collection Data Set"""

    BUILDER_CONFIGS = [
        datasets.BuilderConfig(
            name="plain_text",
            version=datasets.Version("1.0.0", ""),
            description="Plain text import of SMS Spam Collection Data Set",
        )
    ]

    def _info(self):
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(
                {
                    "sms": datasets.Value("string"),
                    "label": datasets.features.ClassLabel(names=["ham", "spam"]),
                }
            ),
            supervised_keys=("sms", "label"),
            homepage="http://archive.ics.uci.edu/ml/datasets/SMS+Spam+Collection",
            citation=_CITATION,
            task_templates=[TextClassification(text_column="sms", label_column="label")],
        )

    def _split_generators(self, dl_manager):
        dl_dir = dl_manager.download_and_extract(_DATA_URL)
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN, gen_kwargs={"filepath": os.path.join(dl_dir, "SMSSpamCollection")}
            ),
        ]

    def _generate_examples(self, filepath):
        """This function returns the examples in the raw (text) form."""

        with open(filepath, encoding="utf-8") as sms_file:
            for idx, line in enumerate(sms_file):
                fields = line.split("\t")

                if fields[0] == "ham":
                    label = 0
                else:
                    label = 1

                yield idx, {
                    "sms": fields[1],
                    "label": label,
                }



"""TODO(break_data): Add a description here."""


import csv
import json
import os
import textwrap

import datasets


# TODO(break): BibTeX citation
_CITATION = """\
@article{Wolfson2020Break,
  title={Break It Down: A Question Understanding Benchmark},
  author={Wolfson, Tomer and Geva, Mor and Gupta, Ankit and Gardner, Matt and Goldberg, Yoav and Deutch, Daniel and Berant, Jonathan},
  journal={Transactions of the Association for Computational Linguistics},
  year={2020},
}
"""

# TODO(break):
_DESCRIPTION = """\
Break is a human annotated dataset of natural language questions and their Question Decomposition Meaning Representations
(QDMRs). Break consists of 83,978 examples sampled from 10 question answering datasets over text, images and databases.
This repository contains the Break dataset along with information on the exact data format.
"""
_URL = "https://github.com/allenai/Break/raw/master/break_dataset/Break-dataset.zip"


class BreakDataConfig(datasets.BuilderConfig):

    """BuilderConfig for Break"""

    def __init__(self, text_features, lexicon_tokens, **kwargs):
        """

        Args:
            text_features: `dict[string, string]`, map from the name of the feature
        dict for each text field to the name of the column in the tsv file
            lexicon_tokens: to define if we want to load the lexicon_tokens files or not
            **kwargs: keyword arguments forwarded to super.
        """
        super(BreakDataConfig, self).__init__(version=datasets.Version("1.0.0", ""), **kwargs)
        self.text_features = text_features
        self.lexicon_tokens = lexicon_tokens


class BreakData(datasets.GeneratorBasedBuilder):
    """TODO(break_data): Short description of my dataset."""

    # TODO(break_data): Set up version.
    VERSION = datasets.Version("0.1.0")
    BUILDER_CONFIGS = [
        BreakDataConfig(
            name="QDMR-high-level",
            description=textwrap.dedent(
                """
             Contains questions annotated with the high-level variant of QDMR. These decomposition are exclusive to Reading
             Comprehension tasks (Section 2). lexicon_tokens files are also provided."""
            ),
            text_features={
                "question_id": "question_id",
                "question_text": "question_text",
                "decomposition": "decomposition",
                "operators": "operators",
                "split": "split",
            },
            lexicon_tokens=False,
        ),
        BreakDataConfig(
            name="QDMR-high-level-lexicon",
            description=textwrap.dedent(
                """
               Contains questions annotated with the high-level variant of QDMR. These decomposition are exclusive to Reading
               Comprehension tasks (Section 2). lexicon_tokens files are also provided."""
            ),
            text_features={
                "source": "source",
                "allowed_tokens": "allowed_tokens",
            },
            lexicon_tokens=True,
        ),
        BreakDataConfig(
            name="QDMR",
            description=textwrap.dedent(
                """
               Contains questions over text, images and databases annotated with their Question Decomposition Meaning
               Representation. In addition to the train, dev and (hidden) test sets we provide lexicon_tokens files. For
               each question, the lexicon file contains the set of valid tokens that could potentially appear in its
               decomposition """
            ),
            text_features={
                "question_id": "question_id",
                "question_text": "question_text",
                "decomposition": "decomposition",
                "operators": "operators",
                "split": "split",
            },
            lexicon_tokens=False,
        ),
        BreakDataConfig(
            name="QDMR-lexicon",
            description=textwrap.dedent(
                """
                 Contains questions over text, images and databases annotated with their Question Decomposition Meaning
               Representation. In addition to the train, dev and (hidden) test sets we provide lexicon_tokens files. For
               each question, the lexicon file contains the set of valid tokens that could potentially appear in its
               decomposition """
            ),
            text_features={
                "source": "source",
                "allowed_tokens": "allowed_tokens",
            },
            lexicon_tokens=True,
        ),
        BreakDataConfig(
            name="logical-forms",
            description=textwrap.dedent(
                """
               Contains questions and QDMRs annotated with full logical-forms of QDMR operators + arguments. Full logical-forms
               were inferred by the annotation-consistency algorithm described in """
            ),
            lexicon_tokens=False,
            text_features={
                "question_id": "question_id",
                "question_text": "question_text",
                "decomposition": "decomposition",
                "operators": "operators",
                "split": "split",
                "program": "program",
            },
        ),
    ]

    def _info(self):
        # TODO(break_data): Specifies the datasets.DatasetInfo object
        features = {text_feature: datasets.Value("string") for text_feature in self.config.text_features.keys()}
        return datasets.DatasetInfo(
            # This is the description that will appear on the datasets page.
            description=_DESCRIPTION,
            # datasets.features.FeatureConnectors
            features=datasets.Features(
                features
                # These are the features of your dataset like images, labels ...
            ),
            # If there's a common (input, target) tuple from the features,
            # specify them here. They'll be used if as_supervised=True in
            # builder.as_dataset.
            supervised_keys=None,
            # Homepage of the dataset for documentation
            homepage="https://github.com/allenai/Break",
            citation=_CITATION,
        )
        # if

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""
        # TODO(break_data): Downloads the data and defines the splits
        # dl_manager is a datasets.download.DownloadManager that can be used to
        # download and extract URLs
        dl_dir = dl_manager.download_and_extract(_URL)
        data_dir = os.path.join(dl_dir, "Break-dataset")
        qdmr_high_level = os.path.join(data_dir, "QDMR-high-level")
        qdmr = os.path.join(data_dir, "QDMR")
        logical = os.path.join(data_dir, "logical-forms")
        if self.config.name == "QDMR" or self.config.name == "QDMR-lexicon":
            return [
                datasets.SplitGenerator(
                    name=datasets.Split.TRAIN,
                    # These kwargs will be passed to _generate_examples
                    gen_kwargs={
                        "filepath": os.path.join(qdmr, "train.csv")
                        if not self.config.lexicon_tokens
                        else os.path.join(qdmr, "train_lexicon_tokens.json")
                    },
                ),
                datasets.SplitGenerator(
                    name=datasets.Split.VALIDATION,
                    # These kwargs will be passed to _generate_examples
                    gen_kwargs={
                        "filepath": os.path.join(qdmr, "dev.csv")
                        if not self.config.lexicon_tokens
                        else os.path.join(qdmr, "dev_lexicon_tokens.json")
                    },
                ),
                datasets.SplitGenerator(
                    name=datasets.Split.TEST,
                    # These kwargs will be passed to _generate_examples
                    gen_kwargs={
                        "filepath": os.path.join(qdmr, "test.csv")
                        if not self.config.lexicon_tokens
                        else os.path.join(qdmr, "test_lexicon_tokens.json")
                    },
                ),
            ]
        elif self.config.name == "QDMR-high-level" or self.config.name == "QDMR-high-level-lexicon":
            return [
                datasets.SplitGenerator(
                    name=datasets.Split.TRAIN,
                    # These kwargs will be passed to _generate_examples
                    gen_kwargs={
                        "filepath": os.path.join(qdmr_high_level, "train.csv")
                        if not self.config.lexicon_tokens
                        else os.path.join(qdmr_high_level, "train_lexicon_tokens.json")
                    },
                ),
                datasets.SplitGenerator(
                    name=datasets.Split.VALIDATION,
                    # These kwargs will be passed to _generate_examples
                    gen_kwargs={
                        "filepath": os.path.join(qdmr_high_level, "dev.csv")
                        if not self.config.lexicon_tokens
                        else os.path.join(qdmr_high_level, "dev_lexicon_tokens.json")
                    },
                ),
                datasets.SplitGenerator(
                    name=datasets.Split.TEST,
                    # These kwargs will be passed to _generate_examples
                    gen_kwargs={
                        "filepath": os.path.join(qdmr_high_level, "test.csv")
                        if not self.config.lexicon_tokens
                        else os.path.join(qdmr_high_level, "test_lexicon_tokens.json")
                    },
                ),
            ]
        elif self.config.name == "logical-forms":
            return [
                datasets.SplitGenerator(
                    name=datasets.Split.TRAIN,
                    # These kwargs will be passed to _generate_examples
                    gen_kwargs={"filepath": os.path.join(logical, "train.csv")},
                ),
                datasets.SplitGenerator(
                    name=datasets.Split.VALIDATION,
                    # These kwargs will be passed to _generate_examples
                    gen_kwargs={"filepath": os.path.join(logical, "dev.csv")},
                ),
                datasets.SplitGenerator(
                    name=datasets.Split.TEST,
                    # These kwargs will be passed to _generate_examples
                    gen_kwargs={"filepath": os.path.join(logical, "test.csv")},
                ),
            ]

    def _generate_examples(self, filepath):
        """Yields examples."""
        # TODO(break_data): Yields (key, example) tuples from the dataset
        with open(filepath, encoding="utf-8") as f:
            if (
                self.config.name == "QDMR-high-level"
                or self.config.name == "QDMR"
                or self.config.name == "logical-forms"
            ):
                data = csv.DictReader(f)
                for id_, row in enumerate(data):
                    yield id_, row
            elif self.config.name == "QDMR-high-level-lexicon" or self.config.name == "QDMR-lexicon":
                for id_, row in enumerate(f):
                    data = json.loads(row)
                    yield id_, data



# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Comments from Jigsaw Toxic Comment Classification Kaggle Competition """


import os

import pandas as pd

import datasets


_DESCRIPTION = """\
This dataset consists of a large number of Wikipedia comments which have been labeled by human raters for toxic behavior.
"""

_HOMEPAGE = "https://www.kaggle.com/c/jigsaw-toxic-comment-classification-challenge/data"

_LICENSE = 'The "Toxic Comment Classification" dataset is released under CC0, with the underlying comment text being governed by Wikipedia\'s CC-SA-3.0.'


class JigsawToxicityPred(datasets.GeneratorBasedBuilder):
    """This is a dataset of comments from Wikipedia’s talk page edits which have been labeled by human raters for toxic behavior."""

    VERSION = datasets.Version("1.1.0")

    @property
    def manual_download_instructions(self):
        return """\
            To use jigsaw_toxicity_pred you have to download it manually from Kaggle: https://www.kaggle.com/c/jigsaw-toxic-comment-classification-challenge/data
            You can manually download the data from it's homepage or use the Kaggle CLI tool (follow the instructions here: https://www.kaggle.com/docs/api)
            Please extract all files in one folder and then load the dataset with:
            `datasets.load_dataset('jigsaw_toxicity_pred', data_dir='/path/to/extracted/data/')`"""

    def _info(self):

        return datasets.DatasetInfo(
            # This is the description that will appear on the datasets page.
            description=_DESCRIPTION,
            # This defines the different columns of the dataset and their types
            features=datasets.Features(
                {
                    "comment_text": datasets.Value("string"),
                    "toxic": datasets.ClassLabel(names=["false", "true"]),
                    "severe_toxic": datasets.ClassLabel(names=["false", "true"]),
                    "obscene": datasets.ClassLabel(names=["false", "true"]),
                    "threat": datasets.ClassLabel(names=["false", "true"]),
                    "insult": datasets.ClassLabel(names=["false", "true"]),
                    "identity_hate": datasets.ClassLabel(names=["false", "true"]),
                }
            ),
            # If there's a common (input, target) tuple from the features,
            # specify them here. They'll be used if as_supervised=True in
            # builder.as_dataset.
            supervised_keys=None,
            # Homepage of the dataset for documentation
            homepage=_HOMEPAGE,
            # License for the dataset if available
            license=_LICENSE,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""
        # This method is tasked with downloading/extracting the data and defining the splits depending on the configuration
        # If several configurations are possible (listed in BUILDER_CONFIGS), the configuration selected by the user is in self.config.name

        data_dir = os.path.abspath(os.path.expanduser(dl_manager.manual_dir))

        if not os.path.exists(data_dir):
            raise FileNotFoundError(
                f"{data_dir} does not exist. Make sure you insert a manual dir via `datasets.load_dataset('jigsaw_toxicity_pred', data_dir=...)`. Manual download instructions: {self.manual_download_instructions}"
            )

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                # These kwargs will be passed to _generate_examples
                gen_kwargs={"train_path": os.path.join(data_dir, "train.csv"), "split": "train"},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                # These kwargs will be passed to _generate_examples
                gen_kwargs={
                    "test_text_path": os.path.join(data_dir, "test.csv"),
                    "test_labels_path": os.path.join(data_dir, "test_labels.csv"),
                    "split": "test",
                },
            ),
        ]

    def _generate_examples(self, split="train", train_path=None, test_text_path=None, test_labels_path=None):
        """Yields examples."""
        # This method will receive as arguments the `gen_kwargs` defined in the previous `_split_generators` method.
        # It is in charge of opening the given file and yielding (key, example) tuples from the dataset
        # The key is not important, it's more here for legacy reason (legacy from tfds)

        if split == "test":
            df1 = pd.read_csv(test_text_path)
            df2 = pd.read_csv(test_labels_path)
            df3 = df1.merge(df2)
            df4 = df3[df3["toxic"] != -1]

        elif split == "train":
            df4 = pd.read_csv(train_path)

        for _, row in df4.iterrows():
            example = {}
            example["comment_text"] = row["comment_text"]

            for label in ["toxic", "severe_toxic", "obscene", "threat", "insult", "identity_hate"]:
                if row[label] != -1:
                    example[label] = int(row[label])
            yield (row["id"], example)



# coding=utf-8
# Copyright 2020 HuggingFace Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Lint as: python3

import datasets


_DESCRIPTION = """\
SETimes – A Parallel Corpus of English and South-East European Languages
The corpus is based on the content published on the SETimes.com news portal. The news portal publishes “news and views from Southeast Europe” in ten languages: Bulgarian, Bosnian, Greek, English, Croatian, Macedonian, Romanian, Albanian and Serbian. This version of the corpus tries to solve the issues present in an older version of the corpus (published inside OPUS, described in the LREC 2010 paper by Francis M. Tyers and Murat Serdar Alperen). The following procedures were applied to resolve existing issues:

- stricter extraction process – no HTML residues present
- language identification on every non-English document – non-English online documents contain English material in case the article was not translated into that language
- resolving encoding issues in Croatian and Serbian – diacritics were partially lost due to encoding errors – text was rediacritized.
"""
_HOMEPAGE_URL = "http://nlp.ffzg.hr/resources/corpora/setimes/"
_CITATION = None

_VERSION = "1.0.0"
_BASE_NAME = "setimes.{}-{}.{}.txt"
_BASE_URL = "http://nlp.ffzg.hr/data/corpora/setimes/setimes.{}-{}.txt.tgz"

_LANGUAGE_PAIRS = [
    ("bg", "bs"),
    ("bg", "el"),
    ("bs", "el"),
    ("bg", "en"),
    ("bs", "en"),
    ("el", "en"),
    ("bg", "hr"),
    ("bs", "hr"),
    ("el", "hr"),
    ("en", "hr"),
    ("bg", "mk"),
    ("bs", "mk"),
    ("el", "mk"),
    ("en", "mk"),
    ("hr", "mk"),
    ("bg", "ro"),
    ("bs", "ro"),
    ("el", "ro"),
    ("en", "ro"),
    ("hr", "ro"),
    ("mk", "ro"),
    ("bg", "sq"),
    ("bs", "sq"),
    ("el", "sq"),
    ("en", "sq"),
    ("hr", "sq"),
    ("mk", "sq"),
    ("ro", "sq"),
    ("bg", "sr"),
    ("bs", "sr"),
    ("el", "sr"),
    ("en", "sr"),
    ("hr", "sr"),
    ("mk", "sr"),
    ("ro", "sr"),
    ("sq", "sr"),
    ("bg", "tr"),
    ("bs", "tr"),
    ("el", "tr"),
    ("en", "tr"),
    ("hr", "tr"),
    ("mk", "tr"),
    ("ro", "tr"),
    ("sq", "tr"),
    ("sr", "tr"),
]


class SetimesConfig(datasets.BuilderConfig):
    def __init__(self, *args, lang1=None, lang2=None, **kwargs):
        super().__init__(
            *args,
            name=f"{lang1}-{lang2}",
            **kwargs,
        )
        self.lang1 = lang1
        self.lang2 = lang2


class Setimes(datasets.GeneratorBasedBuilder):
    BUILDER_CONFIGS = [
        SetimesConfig(
            lang1=lang1,
            lang2=lang2,
            description=f"Translating {lang1} to {lang2} or vice versa",
            version=datasets.Version(_VERSION),
        )
        for lang1, lang2 in _LANGUAGE_PAIRS
    ]
    BUILDER_CONFIG_CLASS = SetimesConfig

    def _info(self):
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(
                {
                    "id": datasets.Value("string"),
                    "translation": datasets.Translation(languages=(self.config.lang1, self.config.lang2)),
                },
            ),
            supervised_keys=None,
            homepage=_HOMEPAGE_URL,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        def _base_url(lang1, lang2):
            return _BASE_URL.format(lang1, lang2)

        download_url = _base_url(self.config.lang1, self.config.lang2)
        archive = dl_manager.download(download_url)
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "l1_files": dl_manager.iter_archive(archive),
                    "l2_files": dl_manager.iter_archive(archive),
                },
            )
        ]

    def _generate_examples(self, l1_files, l2_files):
        l1, l2 = self.config.lang1, self.config.lang2
        l1_file = _BASE_NAME.format(l1, l2, l1)
        l2_file = _BASE_NAME.format(l1, l2, l2)
        for path1, f1 in l1_files:
            if path1 == l1_file:
                for path2, f2 in l2_files:
                    if path2 == l2_file:
                        for sentence_counter, (x, y) in enumerate(zip(f1, f2)):
                            x = x.decode("utf-8").strip()
                            y = y.decode("utf-8").strip()
                            result = (
                                sentence_counter,
                                {
                                    "id": str(sentence_counter),
                                    "translation": {l1: x, l2: y},
                                },
                            )
                            yield result
                        break
                break



from .logging import get_logger


logger = get_logger(__name__)


class _PatchedModuleObj:
    """Set all the modules components as attributes of the _PatchedModuleObj object."""

    def __init__(self, module, attrs=None):
        attrs = attrs or []
        if module is not None:
            for key in getattr(module, "__all__", module.__dict__):
                if key in attrs or not key.startswith("__"):
                    setattr(self, key, getattr(module, key))


class patch_submodule:
    """
    Patch a submodule attribute of an object, by keeping all other submodules intact at all levels.

    Examples:

        >>> import importlib
        >>> from datasets.load import dataset_module_factory
        >>> from datasets.streaming import patch_submodule, xjoin
        >>>
        >>> dataset_module = dataset_module_factory("snli")
        >>> snli_module = importlib.import_module(dataset_module.module_path)
        >>> patcher = patch_submodule(snli_module, "os.path.join", xjoin)
        >>> patcher.start()
        >>> assert snli_module.os.path.join is xjoin
    """

    _active_patches = []

    def __init__(self, obj, target: str, new, attrs=None):
        self.obj = obj
        self.target = target
        self.new = new
        self.key = target.split(".")[0]
        self.original = getattr(obj, self.key, None)
        self.attrs = attrs or []

    def __enter__(self):
        *submodules, attr = self.target.split(".")
        current = self.obj
        for key in submodules:
            setattr(current, key, _PatchedModuleObj(getattr(current, key, None), attrs=self.attrs))
            current = getattr(current, key)
        setattr(current, attr, self.new)

    def __exit__(self, *exc_info):
        setattr(self.obj, self.key, self.original)

    def start(self):
        """Activate a patch."""
        self.__enter__()
        self._active_patches.append(self)

    def stop(self):
        """Stop an active patch."""
        try:
            self._active_patches.remove(self)
        except ValueError:
            # If the patch hasn't been started this will fail
            return None

        return self.__exit__()



# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Portuguese dataset for hate speech detection composed of 5,668 tweets with binary annotations (i.e. 'hate' vs. 'no-hate')."""


import csv

import datasets


_CITATION = """\
@inproceedings{fortuna-etal-2019-hierarchically,
title = "A Hierarchically-Labeled {P}ortuguese Hate Speech Dataset",
author = "Fortuna, Paula  and
    Rocha da Silva, Jo{\\~a}o  and
    Soler-Company, Juan  and
    Wanner, Leo  and
    Nunes, S{\'e}rgio",
booktitle = "Proceedings of the Third Workshop on Abusive Language Online",
month = aug,
year = "2019",
address = "Florence, Italy",
publisher = "Association for Computational Linguistics",
url = "https://www.aclweb.org/anthology/W19-3510",
doi = "10.18653/v1/W19-3510",
pages = "94--104",
abstract = "Over the past years, the amount of online offensive speech has been growing steadily. To successfully cope with it, machine learning are applied. However, ML-based techniques require sufficiently large annotated datasets. In the last years, different datasets were published, mainly for English. In this paper, we present a new dataset for Portuguese, which has not been in focus so far. The dataset is composed of 5,668 tweets. For its annotation, we defined two different schemes used by annotators with different levels of expertise. Firstly, non-experts annotated the tweets with binary labels ({`}hate{'} vs. {`}no-hate{'}). Secondly, expert annotators classified the tweets following a fine-grained hierarchical multiple label scheme with 81 hate speech categories in total. The inter-annotator agreement varied from category to category, which reflects the insight that some types of hate speech are more subtle than others and that their detection depends on personal perception. This hierarchical annotation scheme is the main contribution of the presented work, as it facilitates the identification of different types of hate speech and their intersections. To demonstrate the usefulness of our dataset, we carried a baseline classification experiment with pre-trained word embeddings and LSTM on the binary classified data, with a state-of-the-art outcome.",
}
"""

_DESCRIPTION = """\
Portuguese dataset for hate speech detection composed of 5,668 tweets with binary annotations (i.e. 'hate' vs. 'no-hate').
"""

_HOMEPAGE = "https://github.com/paulafortuna/Portuguese-Hate-Speech-Dataset"

_LICENSE = "Unknown"

_URL = "https://github.com/paulafortuna/Portuguese-Hate-Speech-Dataset/raw/master/2019-05-28_portuguese_hate_speech_binary_classification.csv"


class HateSpeechPortuguese(datasets.GeneratorBasedBuilder):
    """Portuguese dataset for hate speech detection composed of 5,668 tweets with binary annotations (i.e. 'hate' vs. 'no-hate')."""

    VERSION = datasets.Version("1.0.0")

    def _info(self):
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(
                {
                    "text": datasets.Value("string"),
                    "label": datasets.ClassLabel(names=["no-hate", "hate"]),
                    "hatespeech_G1": datasets.Value("string"),
                    "annotator_G1": datasets.Value("string"),
                    "hatespeech_G2": datasets.Value("string"),
                    "annotator_G2": datasets.Value("string"),
                    "hatespeech_G3": datasets.Value("string"),
                    "annotator_G3": datasets.Value("string"),
                }
            ),
            supervised_keys=("text", "label"),
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""

        data_file = dl_manager.download_and_extract(_URL)
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": data_file,
                },
            ),
        ]

    def _generate_examples(self, filepath):
        """Yields examples."""

        with open(filepath, encoding="utf-8") as f:
            reader = csv.reader(f)
            for id_, row in enumerate(reader):
                if id_ == 0:
                    continue

                yield id_, {
                    "text": row[0],
                    "label": "hate" if row[1] == "1" else "no-hate",
                    "hatespeech_G1": row[2],
                    "annotator_G1": row[3],
                    "hatespeech_G2": row[4],
                    "annotator_G2": row[5],
                    "hatespeech_G3": row[6],
                    "annotator_G3": row[7],
                }



# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""NKJP-NER"""


import csv
import os

import datasets
from datasets.tasks import TextClassification


_CITATION = """\
@book{przepiorkowski2012narodowy,
title={Narodowy korpus jezyka polskiego},
author={Przepi{\'o}rkowski, Adam},
year={2012},
publisher={Naukowe PWN}
}
"""

_DESCRIPTION = """\
The NKJP-NER is based on a human-annotated part of National Corpus of Polish (NKJP). We extracted sentences with named entities of exactly one type. The task is to predict the type of the named entity.
"""

_HOMEPAGE = "https://klejbenchmark.com/tasks/"

_LICENSE = "GNU GPL v.3"

_URLs = "https://klejbenchmark.com/static/data/klej_nkjp-ner.zip"


class NkjpNer(datasets.GeneratorBasedBuilder):
    """NKJP-NER"""

    VERSION = datasets.Version("1.1.0")

    def _info(self):
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(
                {
                    "sentence": datasets.Value("string"),
                    "target": datasets.ClassLabel(
                        names=[
                            "geogName",
                            "noEntity",
                            "orgName",
                            "persName",
                            "placeName",
                            "time",
                        ]
                    ),
                }
            ),
            supervised_keys=None,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
            task_templates=[TextClassification(text_column="sentence", label_column="target")],
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""
        data_dir = dl_manager.download_and_extract(_URLs)
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": os.path.join(data_dir, "train.tsv"),
                    "split": "train",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={"filepath": os.path.join(data_dir, "test_features.tsv"), "split": "test"},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={
                    "filepath": os.path.join(data_dir, "dev.tsv"),
                    "split": "dev",
                },
            ),
        ]

    def _generate_examples(self, filepath, split):
        """Yields examples."""
        with open(filepath, encoding="utf-8") as f:
            reader = csv.DictReader(f, delimiter="\t", quoting=csv.QUOTE_NONE)
            for id_, row in enumerate(reader):
                yield id_, {
                    "sentence": row["sentence"],
                    "target": -1 if split == "test" else row["target"],
                }



"""TODO(coqa): Add a description here."""


import json

import datasets


# TODO(coqa): BibTeX citation
_CITATION = """\
@InProceedings{SivaAndAl:Coca,
       author = {Siva, Reddy and Danqi, Chen and  Christopher D., Manning},
        title = {WikiQA: A Challenge Dataset for Open-Domain Question Answering},
      journal = { arXiv},
         year = {2018},

}
"""

# TODO(coqa):
_DESCRIPTION = """\
CoQA: A Conversational Question Answering Challenge
"""

_TRAIN_DATA_URL = "https://nlp.stanford.edu/data/coqa/coqa-train-v1.0.json"
_DEV_DATA_URL = "https://nlp.stanford.edu/data/coqa/coqa-dev-v1.0.json"


class Coqa(datasets.GeneratorBasedBuilder):
    """TODO(coqa): Short description of my dataset."""

    # TODO(coqa): Set up version.
    VERSION = datasets.Version("1.0.0")

    def _info(self):
        # TODO(coqa): Specifies the datasets.DatasetInfo object
        return datasets.DatasetInfo(
            # This is the description that will appear on the datasets page.
            description=_DESCRIPTION,
            # datasets.features.FeatureConnectors
            features=datasets.Features(
                {
                    "source": datasets.Value("string"),
                    "story": datasets.Value("string"),
                    "questions": datasets.features.Sequence(datasets.Value("string")),
                    "answers": datasets.features.Sequence(
                        {
                            "input_text": datasets.Value("string"),
                            "answer_start": datasets.Value("int32"),
                            "answer_end": datasets.Value("int32"),
                        }
                    ),
                }
            ),
            # If there's a common (input, target) tuple from the features,
            # specify them here. They'll be used if as_supervised=True in
            # builder.as_dataset.
            supervised_keys=None,
            # Homepage of the dataset for documentation
            homepage="https://stanfordnlp.github.io/coqa/",
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""
        # TODO(coqa): Downloads the data and defines the splits
        # dl_manager is a datasets.download.DownloadManager that can be used to
        # download and extract URLs
        urls_to_download = {"train": _TRAIN_DATA_URL, "dev": _DEV_DATA_URL}
        downloaded_files = dl_manager.download_and_extract(urls_to_download)

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN, gen_kwargs={"filepath": downloaded_files["train"], "split": "train"}
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION, gen_kwargs={"filepath": downloaded_files["dev"], "split": "validation"}
            ),
        ]

    def _generate_examples(self, filepath, split):
        """Yields examples."""
        # TODO(coqa): Yields (key, example) tuples from the dataset
        with open(filepath, encoding="utf-8") as f:
            data = json.load(f)
            for row in data["data"]:
                questions = [question["input_text"] for question in row["questions"]]
                story = row["story"]
                source = row["source"]
                answers_start = [answer["span_start"] for answer in row["answers"]]
                answers_end = [answer["span_end"] for answer in row["answers"]]
                answers = [answer["input_text"] for answer in row["answers"]]
                yield row["id"], {
                    "source": source,
                    "story": story,
                    "questions": questions,
                    "answers": {"input_text": answers, "answer_start": answers_start, "answer_end": answers_end},
                }



# coding=utf-8
# Copyright 2020 The TensorFlow Datasets Authors and the HuggingFace Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Lint as: python3
"""Detecing which tweets showcase hate or racist remarks."""


import csv

import datasets
from datasets.tasks import TextClassification


_DESCRIPTION = """\
The objective of this task is to detect hate speech in tweets. For the sake of simplicity, we say a tweet contains hate speech if it has a racist or sexist sentiment associated with it. So, the task is to classify racist or sexist tweets from other tweets.

Formally, given a training sample of tweets and labels, where label ‘1’ denotes the tweet is racist/sexist and label ‘0’ denotes the tweet is not racist/sexist, your objective is to predict the labels on the given test dataset.
"""

_CITATION = """\
@InProceedings{Z
Roshan Sharma:dataset,
title = {Sentimental Analysis of Tweets for Detecting Hate/Racist Speeches},
authors={Roshan Sharma},
year={2018}
}
"""

_TRAIN_DOWNLOAD_URL = (
    "https://raw.githubusercontent.com/sharmaroshan/Twitter-Sentiment-Analysis/master/train_tweet.csv"
)


class TweetsHateSpeechDetection(datasets.GeneratorBasedBuilder):
    """Detecting which tweets showcase hate or racist remarks."""

    def _info(self):
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(
                {
                    "label": datasets.ClassLabel(names=["no-hate-speech", "hate-speech"]),
                    "tweet": datasets.Value("string"),
                }
            ),
            homepage="https://github.com/sharmaroshan/Twitter-Sentiment-Analysis",
            citation=_CITATION,
            task_templates=[TextClassification(text_column="tweet", label_column="label")],
        )

    def _split_generators(self, dl_manager):
        train_path = dl_manager.download_and_extract(_TRAIN_DOWNLOAD_URL)

        return [
            datasets.SplitGenerator(name=datasets.Split.TRAIN, gen_kwargs={"filepath": train_path}),
        ]

    def _generate_examples(self, filepath):
        """Generate Tweet examples."""
        with open(filepath, encoding="utf-8") as csv_file:
            csv_reader = csv.reader(
                csv_file, quotechar='"', delimiter=",", quoting=csv.QUOTE_ALL, skipinitialspace=True
            )
            next(csv_reader, None)
            for id_, row in enumerate(csv_reader):
                row = row[1:]
                (label, tweet) = row

                yield id_, {
                    "label": int(label),
                    "tweet": (tweet),
                }



# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""TODO: Add a description here."""


import csv
import os

import datasets
from datasets.tasks import TextClassification


# no BibTeX citation
_CITATION = ""

_DESCRIPTION = """\
Wongnai's review dataset contains restaurant reviews and ratings, mainly in Thai language.
The reviews are in 5 classes ranging from 1 to 5 stars.
"""

_LICENSE = "LGPL-3.0"

_URLs = {"default": "https://archive.org/download/wongnai_reviews/wongnai_reviews_withtest.zip"}


class WongnaiReviews(datasets.GeneratorBasedBuilder):
    VERSION = datasets.Version("1.0.1")

    def _info(self):
        features = datasets.Features(
            {
                "review_body": datasets.Value("string"),
                "star_rating": datasets.features.ClassLabel(names=["1", "2", "3", "4", "5"]),
            }
        )
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            supervised_keys=None,
            homepage="https://github.com/wongnai/wongnai-corpus",
            license=_LICENSE,
            citation=_CITATION,
            task_templates=[TextClassification(text_column="review_body", label_column="star_rating")],
        )

    def _split_generators(self, dl_manager):
        my_urls = _URLs[self.config.name]
        data_dir = dl_manager.download_and_extract(my_urls)
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={"filepath": os.path.join(data_dir, "w_review_train.csv"), "split": "train"},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={"filepath": os.path.join(data_dir, "w_review_test.csv"), "split": "test"},
            ),
        ]

    def _generate_examples(self, filepath, split):
        with open(filepath, encoding="utf-8") as f:
            spamreader = csv.reader(f, delimiter=";", quotechar='"')
            for id_, row in enumerate(spamreader):
                yield id_, {"review_body": row[0], "star_rating": row[1]}



# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Recognizing the flow of time in a story is a crucial aspect of understanding it. Prior work related to time has primarily focused on identifying temporal expressions or relative sequencing of events, but here we propose computationally annotating each line of a book with wall clock times, even in the absence of explicit time-descriptive phrases. To do so, we construct a data set of hourly time phrases from 52,183 fictional books."""


import csv
import os

import datasets


_CITATION = """\
@misc{kim2020time,
      title={What time is it? Temporal Analysis of Novels},
      author={Allen Kim and Charuta Pethe and Steven Skiena},
      year={2020},
      eprint={2011.04124},
      archivePrefix={arXiv},
      primaryClass={cs.CL}
}
"""

_DESCRIPTION = """\
A clean data resource containing all explicit time references in a dataset of 52,183 novels whose full text is available via Project Gutenberg.
"""

_HOMEPAGE = "https://github.com/allenkim/what-time-is-it"

_LICENSE = "[More Information needed]"

# The HuggingFace dataset library don't host the datasets but only point to the original files
# This can be an arbitrary nested dict/list of URLs (see below in `_split_generators` method)
_URLs = {
    "gutenberg": "https://github.com/TevenLeScao/what-time-is-it/blob/master/gutenberg_time_phrases.zip?raw=true",
}


class GutenbergTime(datasets.GeneratorBasedBuilder):
    """Novel extracts with time-of-the-day information"""

    VERSION = datasets.Version("1.1.3")
    BUILDER_CONFIGS = [
        datasets.BuilderConfig(name="gutenberg", description="Data pulled from the Gutenberg project"),
    ]

    def _info(self):
        features = datasets.Features(
            {
                "guten_id": datasets.Value("string"),
                "hour_reference": datasets.Value("string"),
                "time_phrase": datasets.Value("string"),
                "is_ambiguous": datasets.Value("bool_"),
                "time_pos_start": datasets.Value("int64"),
                "time_pos_end": datasets.Value("int64"),
                "tok_context": datasets.Value("string"),
            }
        )
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            supervised_keys=None,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""
        my_urls = _URLs[self.config.name]
        data = dl_manager.download_and_extract(my_urls)
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                # These kwargs will be passed to _generate_examples
                gen_kwargs={
                    "filepath": os.path.join(data, "gutenberg_time_phrases.csv"),
                    "split": "train",
                },
            )
        ]

    def _generate_examples(self, filepath, split):

        with open(filepath, encoding="utf8") as f:
            data = csv.reader(f)
            next(data)
            for id_, row in enumerate(data):
                yield id_, {
                    "guten_id": row[0],
                    "hour_reference": row[1],
                    "time_phrase": row[2],
                    "is_ambiguous": row[3],
                    "time_pos_start": row[4],
                    "time_pos_end": row[5],
                    "tok_context": row[6],
                }



# coding=utf-8
# Copyright 2020 HuggingFace Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Lint as: python3
import json

import datasets


_DESCRIPTION = """\
Acronym identification training and development sets for the acronym identification task at SDU@AAAI-21.
"""
_HOMEPAGE_URL = "https://github.com/amirveyseh/AAAI-21-SDU-shared-task-1-AI"
_CITATION = """\
@inproceedings{veyseh-et-al-2020-what,
   title={{What Does This Acronym Mean? Introducing a New Dataset for Acronym Identification and Disambiguation}},
   author={Amir Pouran Ben Veyseh and Franck Dernoncourt and Quan Hung Tran and Thien Huu Nguyen},
   year={2020},
   booktitle={Proceedings of COLING},
   link={https://arxiv.org/pdf/2010.14678v1.pdf}
}
"""

_TRAIN_URL = "https://raw.githubusercontent.com/amirveyseh/AAAI-21-SDU-shared-task-1-AI/master/dataset/train.json"
_VALID_URL = "https://raw.githubusercontent.com/amirveyseh/AAAI-21-SDU-shared-task-1-AI/master/dataset/dev.json"
_TEST_URL = "https://raw.githubusercontent.com/amirveyseh/AAAI-21-SDU-shared-task-1-AI/master/dataset/test.json"


class AcronymIdentification(datasets.GeneratorBasedBuilder):
    VERSION = datasets.Version("1.0.0")

    def _info(self):
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(
                {
                    "id": datasets.Value("string"),
                    "tokens": datasets.Sequence(datasets.Value("string")),
                    "labels": datasets.Sequence(
                        datasets.ClassLabel(names=["B-long", "B-short", "I-long", "I-short", "O"])
                    ),
                },
            ),
            supervised_keys=None,
            homepage=_HOMEPAGE_URL,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        train_path = dl_manager.download_and_extract(_TRAIN_URL)
        valid_path = dl_manager.download_and_extract(_VALID_URL)
        test_path = dl_manager.download_and_extract(_TEST_URL)
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={"datapath": train_path, "datatype": "train"},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={"datapath": valid_path, "datatype": "valid"},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={"datapath": test_path, "datatype": "test"},
            ),
        ]

    def _generate_examples(self, datapath, datatype):
        with open(datapath, encoding="utf-8") as f:
            data = json.load(f)

        for sentence_counter, d in enumerate(data):
            resp = {
                "id": d["id"],
                "tokens": d["tokens"],
            }
            if datatype != "test":
                resp["labels"] = d["labels"]
            else:
                resp["labels"] = ["O"] * len(d["tokens"])
            yield sentence_counter, resp



# coding=utf-8
# Copyright 2020 The TensorFlow Datasets Authors and the HuggingFace Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Lint as: python3
"""Software Applications User Reviews"""


import csv

import datasets


_DESCRIPTION = """\
It is a large dataset of Android applications belonging to 23 differentapps categories, which provides an overview of the types of feedback users report on the apps and documents the evolution of the related code metrics. The dataset contains about 395 applications of the F-Droid repository, including around 600 versions, 280,000 user reviews (extracted with specific text mining approaches)
"""

_CITATION = """\
@InProceedings{Zurich Open Repository and
Archive:dataset,
title = {Software Applications User Reviews},
authors={Grano, Giovanni; Di Sorbo, Andrea; Mercaldo, Francesco; Visaggio, Corrado A; Canfora, Gerardo;
Panichella, Sebastiano},
year={2017}
}
"""

_TRAIN_DOWNLOAD_URL = "https://raw.githubusercontent.com/sealuzh/user_quality/master/csv_files/reviews.csv"


class AppReviews(datasets.GeneratorBasedBuilder):
    """Software Application Reviews by Users."""

    def _info(self):
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(
                {
                    "package_name": datasets.Value("string"),
                    "review": datasets.Value("string"),
                    "date": datasets.Value("string"),
                    "star": datasets.Value("int8"),
                }
            ),
            homepage="https://giograno.me/assets/pdf/workshop/wama17.pdf",
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        train_path = dl_manager.download_and_extract(_TRAIN_DOWNLOAD_URL)
        return [
            datasets.SplitGenerator(name=datasets.Split.TRAIN, gen_kwargs={"filepath": train_path}),
        ]

    def _generate_examples(self, filepath):
        """Generate Distaster Response Messages examples."""
        with open(filepath, encoding="utf-8") as csv_file:
            csv_reader = csv.reader(
                csv_file, quotechar='"', delimiter=",", quoting=csv.QUOTE_ALL, skipinitialspace=True
            )
            next(csv_reader, None)
            for id_, row in enumerate(csv_reader):
                row = row[1:5]
                (package_name, review, date, star) = row

                yield id_, {
                    "package_name": (package_name),
                    "review": (review),
                    "date": (date),
                    "star": int(star),
                }



import copy
from dataclasses import dataclass
from typing import ClassVar, Dict

from ..features import ClassLabel, Features, Value
from .base import TaskTemplate


@dataclass(frozen=True)
class TextClassification(TaskTemplate):
    # `task` is not a ClassVar since we want it to be part of the `asdict` output for JSON serialization
    task: str = "text-classification"
    input_schema: ClassVar[Features] = Features({"text": Value("string")})
    label_schema: ClassVar[Features] = Features({"labels": ClassLabel})
    text_column: str = "text"
    label_column: str = "labels"

    def align_with_features(self, features):
        if self.label_column not in features:
            raise ValueError(f"Column {self.label_column} is not present in features.")
        if not isinstance(features[self.label_column], ClassLabel):
            raise ValueError(f"Column {self.label_column} is not a ClassLabel.")
        task_template = copy.deepcopy(self)
        label_schema = self.label_schema.copy()
        label_schema["labels"] = features[self.label_column]
        task_template.__dict__["label_schema"] = label_schema
        return task_template

    @property
    def column_mapping(self) -> Dict[str, str]:
        return {
            self.text_column: "text",
            self.label_column: "labels",
        }
